
import datetime
from xml.dom import minidom
import os
import subprocess
# https://github.com/J-CITY/tinyWinToast



TEXT_ALIGN_LEFT = "left"
TEXT_ALIGN_RIGHT = "right"

TEXT_STYLE_CAPTION_SUBTLE = "captionSubtle"
TEXT_STYLE_BASE = "base"

CROP_NONE = "none"
CROP_CIRCLE = "circle"

DURATION_SHORT = "short"
DURATION_LONG = "long"

DEFAULT_APP_ID = "TiniWinToastApp"

ACTION_TYPE_PROTOCOL = "protocol"
ACTION_TYPE_BACKGROUND = "background"
ACTION_TYPE_FOREGROUND = "foreground"
ACTION_TYPE_SYSTEM = "system"

SCENARIO_DEFAULT = "default"
SCENARIO_ALARM = "alarm"
SCENARIO_REMINDER = "reminder"
SCENARIO_INCOMING_CALL = "incomingCall"

INPUT_TEXT = "text"
INPUT_SELECTION = "selection"

ACTION_INPUT = "input"
ACTION_BUTTON = "button"

class Config:
	def __init__(self):
		pass

	# for dynamic toast update need set tag and group
	TAG = ""
	GROUP = ""

	#Application id
	APP_ID = DEFAULT_APP_ID

	# foreground | background | protocol | system
	# ACTION_TYPE_PROTOCOL
	# ACTION_TYPE_BACKGROUND
	# ACTION_TYPE_FOREGROUND
	# ACTION_TYPE_SYSTEM
	ACTION_TYPE = ACTION_TYPE_PROTOCOL

	ACTION_ARGUMENT = ""

	# - default | alarm | reminder | incomingCall"
	# SCENARIO_DEFAULT
	# SCENARIO_ALARM
	# SCENARIO_REMINDER
	# SCENARIO_INCOMING_CALL
	SCENARIO = SCENARIO_DEFAULT

	# short - 5sec - DURATION_SHORT
	# long - 25 sec - DURATION_SHORT
	DURATION = DURATION_SHORT

	BASE_URI = ""

	# [0] - TITLE
	# [1] - MESSAGE
	# [2]... other message
	TEXTS = []

	ICO_IMAGE = ()

	HERO_IMAGE = ()

	IMAGE = []

	# in ISO 8601
	TIME = ""

	# true to set current time
	SET_CUT_TIME = False

	PROGRESS = []

	HEADER = None

	TEXTS_GROUP_LEFT = []
	TEXTS_GROUP_RIGHT = []

	_WITH_ACTIONS = False

	ACTIONS = []

	AUDIO=None

class Audio:
	def __init__(self, isSilent=False, src="", isLoop=False):
		"""Add audio to toast

		:isSilent: true for mute toast sound
		:src: audio source
		:isLoop: if you want loop
		"""
		self.isSilent = isSilent
		self.src = src
		self.isLoop = isLoop

class Progress:
	def __init__(self, title="", value="", valueStringOverride="", status=""):
		"""Progress for toast

		:title: progress title
		:value: value of progress bar (0.0 - 1.0)
		:valueStringOverride: value string info
		:status: status
		"""
		self.title = title
		self.value = value
		self.valueStringOverride = valueStringOverride
		self.status = status

class Header:
	def __init__(self, id="", title="", arguments=""):
		"""Set header if you want group toast

		:id: id of your toast group
		:title: header name
		:arguments: action
		"""
		self.id= id
		self.title= title
		self.arguments= arguments

class Button:
	def __init__(self, content="", arguments="", activationType=ACTION_TYPE_FOREGROUND, pendingUpdate=False, inputId="", imageUri=""):
		"""Add button to toast

		:content: button text
		:arguments: button action
		:activationType: /background/foreground
		:pendingUpdate: if True then toast wait update
		:inputId: set id of input box (if you need)
		:imageUri: set image for button
		"""
		self.content = content
		self.arguments = arguments
		self.activationType = activationType
		self.pendingUpdate = pendingUpdate
		self.inputId = inputId
		self.imageUri = imageUri


class Input:
	def __init__(self, inputId="", intype=INPUT_TEXT, placeHolderContent="", selections = [], defaultInput=""):
		"""Add input to toast

		:inputId: input id
		:intype: type /text/selection
		:placeHolderContent: input himt if type is text
		:selections: list of selections (id, content)
		:defaultInput: default selection id if type is selection
		"""
		self.inputId = inputId
		self.intype = intype
		self.placeHolderContent = placeHolderContent
		self.selections = selections
		self.defaultInput = defaultInput

	def addSelection(self, id, content):
		selection.append(id, content)

class Toast:
	def __init__(self, config=None):
		"""Init

		:config: toast config
		"""
		if config is None:
			self.config = Config()
		else:
			self.config = config

	def setConfig(self, config):
		"""Set toast config

		:config: toast config
		"""
		self.config = config

	def getConfig(self):
		"""Get toast config"""
		return self.config

	def setAppID(self, appid):
		"""Set application ID

		"appid" application ID (string)
		"""
		self.config.APP_ID = appid

	def setTag(self, tag):
		"""Set toast tag (if you want grout yout notifications)

		:tag: toast tag
		"""
		self.config.TAG = tag

	def setGroup(self, group):
		"""Set toast group (if you want grout yout notifications)

		:group: toast group
		"""
		self.config.GROUP = group

	def setTime(self, time):
		"""Set toast time

		:time: toast time in ISO 8601 format
		"""
		self.config.TIME = time

	def setCurTime(self):
		"""Set toast current time"""
		self.config.SET_CUT_TIME = True

	def setTitle(self, title, maxLines=1):
		"""Set title

		:title: toast title
		:maxLines: max title lines
		"""
		if len(self.config.TEXTS) > 0:
			self.config.TEXTS[0] = (title, maxLines, "", "", False)
		else:
			self.config.TEXTS.append((title, maxLines, "", "", False))

	def setMessage(self, msg, maxLines=1):
		"""Set message

		:msg: toast message
		:maxLines: max message lines
		"""
		if len(self.config.TEXTS) > 1:
			self.config.TEXTS[1] = (msg, maxLines, "", "", False)
		elif len(self.config.TEXTS) == 0:
			self.config.TEXTS.append(("", 1, ""))
			self.config.TEXTS.append((msg, maxLines, "", "", False))
		else:
			self.config.TEXTS.append((msg, maxLines, "", "", False))

	def addText(self, msg, maxLines=1, placement="", style=TEXT_STYLE_BASE, wrap=False):
		"""Add message to toast

		:msg: message
		:maxLines: max text lines
		:placement: set attribute to text /attribution
		:style: style /captionSubtle/base
		:wrap: wrap text
		"""
		if len(self.config.TEXTS) == 0:
			self.config.TEXTS.append(("", 1, ""))
			self.config.TEXTS.append((msg, maxLines, placement, style, wrap))
		else:
			self.config.TEXTS.append((msg, maxLines, placement, style, wrap))

	def addGroupText(self, msg, maxLines=1, placement="", style=TEXT_STYLE_BASE, wrap=False, align=TEXT_ALIGN_LEFT):
		"""Add text in footer

		:msg: message
		:maxLines: max text lines
		:placement: set attribute to text /attribution
		:style: style /captionSubtle/base
		:wrap: wrap text
		:align: /left/right
		"""
		if align == "left":
			if len(self.config.TEXTS_GROUP_LEFT) == 0:
				self.config.TEXTS_GROUP_LEFT.append(("", 1, "", "captionSubtle", False))
				self.config.TEXTS_GROUP_LEFT.append((msg, maxLines, placement, style, wrap))
			else:
				self.config.TEXTS_GROUP_LEFT.append((msg, maxLines, placement, style, wrap))
		else:
			if len(self.config.TEXTS_GROUP_RIGHT) == 0:
				self.config.TEXTS_GROUP_RIGHT.append(("", 1, "", "captionSubtle", False))
				self.config.TEXTS_GROUP_RIGHT.append((msg, maxLines, placement, style, wrap))
			else:
				self.config.TEXTS_GROUP_RIGHT.append((msg, maxLines, placement, style, wrap))

	def setIcon(self, src, crop=CROP_NONE):
		"""Set toast logo

		:src: path or url to image
		:crop: image crop /none/circle
		"""
		self.config.ICO_IMAGE = ("appLogoOverride", src, crop)

	def setHeroImage(self, src):
		"""Set hero image

		:src: path or url to image
		"""
		self.config.HERO_IMAGE = ("hero", src)

	def setImage(self, src):
		"""Set image

		:src: path or url to image
		"""
		self.config.IMAGE = [src]

	def addImage(self, src):
		"""Set image

		:src: path or url to image
		"""
		self.config.IMAGE.append(src)

	def setDuration(self, duration="short"):
		self.config.DURATION = duration

	def addHeader(self, header):
		"""Add toast header

		:header: Head object
		"""
		self.config.HEADER = header

	def addProgress(self, progress):
		"""Add progress data"""
		self.config.PROGRESS.append(progress)

	def setProgress(self, progress):
		"""Set progers data and clear old data"""
		self.config.PROGRESS = [progress]

	def addButton(self, button):
		"""Add button to toast

		:button: Button object
		"""
		self.config._WITH_ACTIONS = True
		self.config.ACTIONS.append(("button", button))

	def addInput(self, input):
		"""Add input to toast

		:input: Input object
		"""
		self.config._WITH_ACTIONS = True
		self.config.ACTIONS.append(("input", input))

	def setAudio(self, audio):
		"""Set toast sound

		"audio": audio object
		"""
		self.config.AUDIO = audio

	def __genXML(self):
		"""Generate xml"""
		head = """
[Windows.UI.Notifications.ToastNotificationManager, Windows.UI.Notifications, ContentType = WindowsRuntime] | Out-Null
[Windows.UI.Notifications.ToastNotification, Windows.UI.Notifications, ContentType = WindowsRuntime] | Out-Null
[Windows.Data.Xml.Dom.XmlDocument, Windows.Data.Xml.Dom.XmlDocument, ContentType = WindowsRuntime] | Out-Null
$APP_ID = '{0}'""".format(self.config.APP_ID)
		if self.config.TAG != "":
			head += "\n$tag = '{0}'\n".format(self.config.TAG)

		if self.config.GROUP != "":
			head += "\n$group = '{0}'\n".format(self.config.GROUP)
		head += '\n$template = @"\n'


		doc = minidom.Document()
		toast = doc.createElement('toast')
		doc.appendChild(toast)
		toast.setAttribute("activationType", self.config.ACTION_TYPE)
		toast.setAttribute("launch", self.config.ACTION_ARGUMENT)
		toast.setAttribute("duration", self.config.DURATION)
		toast.setAttribute("scenario", self.config.SCENARIO)
		if self.config.TIME != '':
			toast.setAttribute("displayTimestamp", self.config.TIME)
		elif self.config.SET_CUT_TIME:
			self.config.TIME = str(datetime.datetime.now())
			self.config.TIME = self.config.TIME.replace(' ', 'T')
			import re
			self.config.TIME = re.sub(r"\.\d+", ".00Z", self.config.TIME)
			toast.setAttribute("displayTimestamp", self.config.TIME)

		if not self.config.HEADER is None:
			header = doc.createElement('header')
			header.setAttribute("id", self.config.HEADER.id)
			header.setAttribute("title", self.config.HEADER.title)
			header.setAttribute("arguments", self.config.HEADER.arguments)
			toast.appendChild(header)

		visual = doc.createElement('visual')
		toast.appendChild(visual)

		binding = doc.createElement('binding')
		visual.appendChild(binding)
		binding.setAttribute("template", "ToastGeneric")

		for txt in self.config.TEXTS:
			text = doc.createElement('text')
			data = doc.createTextNode(txt[0])
			text.appendChild(data)
			text.setAttribute("hint-maxLines", str(txt[1]))
			if txt[2] != '':
				text.setAttribute("placement", str(txt[2]))
			if txt[3] != '':
				text.setAttribute("hint-style", str(txt[3]))
			if txt[4]:
				text.setAttribute("hint-wrap", "true")
			binding.appendChild(text)

		if self.config.ICO_IMAGE != ():
			image = doc.createElement('image')
			image.setAttribute("placement", self.config.ICO_IMAGE[0])
			image.setAttribute("hint-crop", self.config.ICO_IMAGE[2])
			image.setAttribute("src", self.config.ICO_IMAGE[1])
			binding.appendChild(image)

		if self.config.HERO_IMAGE != ():
			image = doc.createElement('image')
			image.setAttribute("placement", self.config.HERO_IMAGE[0])
			image.setAttribute("src", self.config.HERO_IMAGE[1])
			binding.appendChild(image)

		if self.config.IMAGE != []:
			for im in self.config.IMAGE:
				image = doc.createElement('image')
				image.setAttribute("src", im)
				binding.appendChild(image)


		for p in self.config.PROGRESS:
			progress = doc.createElement('progress')
			progress.setAttribute("title", p.title)
			progress.setAttribute("value", p.value)
			progress.setAttribute("valueStringOverride", p.valueStringOverride)
			progress.setAttribute("status", p.status)
			binding.appendChild(progress)

		if self.config.TEXTS_GROUP_LEFT != [] or self.config.TEXTS_GROUP_RIGHT != []:
			group = doc.createElement('group')
			binding.appendChild(group)

			subgroup1 = doc.createElement('subgroup')
			subgroup2 = doc.createElement('subgroup')
			group.appendChild(subgroup1)
			group.appendChild(subgroup2)
			for txt in self.config.TEXTS_GROUP_LEFT:
				text = doc.createElement('text')
				data = doc.createTextNode(txt[0])
				text.appendChild(data)
				text.setAttribute("hint-maxLines", str(txt[1]))
				if txt[2] != '':
					text.setAttribute("placement", str(txt[2]))
				if txt[3] != '':
					text.setAttribute("hint-style", str(txt[3]))
				if txt[4]:
					text.setAttribute("hint-wrap", "true")
				subgroup1.appendChild(text)

			for txt in self.config.TEXTS_GROUP_RIGHT:
				text = doc.createElement('text')
				data = doc.createTextNode(txt[0])
				text.appendChild(data)
				text.setAttribute("hint-maxLines", str(txt[1]))
				if txt[2] != '':
					text.setAttribute("placement", str(txt[2]))
				if txt[3] != '':
					text.setAttribute("hint-style", str(txt[3]))
				if txt[4]:
					text.setAttribute("hint-wrap", "true")
				text.setAttribute("hint-align", "right")
				subgroup2.appendChild(text)

		# actions
		if self.config._WITH_ACTIONS:
			actions = doc.createElement('actions')
			toast.appendChild(actions)

			for a in self.config.ACTIONS:
				t = a[0]

				b = a[1]

				if a[0] == ACTION_BUTTON:
					action = doc.createElement("action")
					action.setAttribute("content", b.content)
					action.setAttribute("arguments", b.arguments)
					action.setAttribute("activationType", b.activationType)
					if b.imageUri != "":
						action.setAttribute("imageUri", b.imageUri)
					if b.inputId != "":
						action.setAttribute("hint-inputId", b.inputId)
					if b.pendingUpdate:
						action.setAttribute("afterActivationBehavior", "pendingUpdate")
					if b.inputId != "":
						action.setAttribute("hint-inputId", b.inputId)
					actions.appendChild(action)
				else:
					input = doc.createElement("input")
					input.setAttribute("id", b.inputId)
					input.setAttribute("type", b.intype)
					if b.placeHolderContent != "" and b.intype == "text":
						input.setAttribute("placeHolderContent", b.placeHolderContent)
					if b.defaultInput != "" and b.intype == "selection":
						input.setAttribute("defaultInput", b.defaultInput)

					if b.intype == 'selection':
						for s in b.selections:
							selection = doc.createElement("selection")
							selection.setAttribute("id", s[0])
							selection.setAttribute("content", s[1])
							input.appendChild(selection)
					actions.appendChild(input)


		if not self.config.AUDIO is None:
			audio = doc.createElement('audio')
			if self.config.AUDIO.isSilent:
				audio.setAttribute("silent", "true")
			else:
				audio.setAttribute("src", self.config.AUDIO.src)
				if self.config.AUDIO.loop:
					audio.setAttribute("loop", "true")
			toast.appendChild(audio)

		tail = '''"@
$xml = New-Object Windows.Data.Xml.Dom.XmlDocument
$xml.LoadXml($template)
$toast = New-Object Windows.UI.Notifications.ToastNotification $xml
'''
		if self.config.TAG != "":
			tail += "$toast.Tag = $tag\n"
		if self.config.GROUP != "":
			tail += "$toast.Group = $group\n"
		tail += '''
[Windows.UI.Notifications.ToastNotificationManager]::CreateToastNotifier($APP_ID)'''

		tail += '.Show($toast)'

		toaststr = head + '\n' + doc.toprettyxml(indent="	").split("\n",1)[1] + '\n' + tail
		return toaststr

	def show(self):
		"""Show toast"""
		toaststr = self.__genXML()
		path = os.path.dirname(os.path.realpath(__file__)) + "\\"
		text_file = open(path + "toast.ps1", "w")
		text_file.write(toaststr)
		text_file.close()
		subprocess.run(["PowerShell", "-ExecutionPolicy", "Bypass", "-File", path + "toast.ps1"])
		os.remove(path + "toast.ps1")

	def update(self, sequenceId, data):
		"""Update toast"""
		toaststr = """
[Windows.UI.Notifications.ToastNotificationManager, Windows.UI.Notifications, ContentType = WindowsRuntime] | Out-Null
[Windows.UI.Notifications.ToastNotification, Windows.UI.Notifications, ContentType = WindowsRuntime] | Out-Null
[Windows.Data.Xml.Dom.XmlDocument, Windows.Data.Xml.Dom.XmlDocument, ContentType = WindowsRuntime] | Out-Null
$APP_ID = '{0}'""".format(self.config.APP_ID)
		toaststr+= """
$sequenceId = {0}
		""".format(sequenceId)
		toaststr+= """
$tag = '{0}'
		""".format(self.config.TAG)
		toaststr+= """
$group = '{0}'
		""".format(self.config.GROUP)
		toaststr+='''

$data = New-Object Windows.UI.Notifications.NotificationData
$data.SequenceNumber = $sequenceId

$DataDictionary = New-Object 'system.collections.generic.dictionary[string,string]'
'''

		for key, val in data.items():
			toaststr += "$DataDictionary.Add('{0}', '{1}')\n".format(key, val)

		toaststr += '''

$ToastData = [Windows.UI.Notifications.NotificationData]::new($DataDictionary)


[Windows.UI.Notifications.ToastNotificationManager]::CreateToastNotifier($APP_ID)'''
		toaststr += ".Update($ToastData, $tag, $group)"
		path = os.path.dirname(os.path.realpath(__file__)) + "\\"
		text_file = open(path + "toast.ps1", "w")
		text_file.write(toaststr)
		text_file.close()
		subprocess.run(["PowerShell", "-ExecutionPolicy", "Bypass", "-File", path + "toast.ps1"])
		os.remove(path + "toast.ps1")

	def showFromXml(self, appId, xml):
		"""Show toast from xml

		:appId: application id
		:xml: xml string
		"""
		toaststr = """
[Windows.UI.Notifications.ToastNotificationManager, Windows.UI.Notifications, ContentType = WindowsRuntime] | Out-Null
[Windows.UI.Notifications.ToastNotification, Windows.UI.Notifications, ContentType = WindowsRuntime] | Out-Null
[Windows.Data.Xml.Dom.XmlDocument, Windows.Data.Xml.Dom.XmlDocument, ContentType = WindowsRuntime] | Out-Null
$APP_ID = '{0}'
$template = @"
{1}
"@
$xml = New-Object Windows.Data.Xml.Dom.XmlDocument
$xml.LoadXml($template)
$toast = New-Object Windows.UI.Notifications.ToastNotification $xml
[Windows.UI.Notifications.ToastNotificationManager]::CreateToastNotifier($APP_ID).Show($toast)""".format(appId, xml)
		path = os.path.dirname(os.path.realpath(__file__)) + "\\"
		text_file = open("toast.ps1", "w")
		text_file.write(toaststr)
		text_file.close()

		subprocess.run(["PowerShell", "-ExecutionPolicy", "Bypass", "-File", path + "toast.ps1"])
		os.remove(path + "toast.ps1")

	def startFromScript(self, scriptPath):
		"""Start toast from script

		:scriptPath: script.ps1 path
		"""
		subprocess.run(["PowerShell", "-ExecutionPolicy", "Bypass", "-File", scriptPath])

	def updateFromScript(self, scriptPath):
		"""Update toast from script

		:scriptPath: script.ps1 path
		"""
		subprocess.run(["PowerShell", "-ExecutionPolicy", "Bypass", "-File", scriptPath])


def getToast(title, message, icon="", iconCrop=CROP_NONE, duration=DURATION_SHORT, appId="", isMute=True):
	"""Create simple toast

	:title: toast title
	:message: toast message
	:icon: path or url to icon
	:iconCrop: crop icon /none/circle
	:duration: duration /short/long
	"""
	toast = Toast()
	if appId != "":
		toast.setAppID(appId)
	toast.setTitle(title, maxLines=1)
	toast.setMessage(message, maxLines=1)
	if icon != "":
		toast.setIcon(icon, crop=iconCrop)
	toast.setDuration(duration)
	if isMute:
		toast.setAudio(Audio(True))
	return toast

#toast = Toast()
#toast.setTitle("TITLE", maxLines=1)
#toast.setMessage("MESSAGE", maxLines=1)
##toast.setTag("mytag")
##toast.setGroup("mygroup")
##toast.addInput(Input(inputId="111", intype="text", placeHolderContent="Input text..."))
##toast.addInput(Input(inputId="112", intype="selection", selections = [("1","Yes"), ("2","No"), ("3","Maybe")], defaultInput="1"))
##toast.addButton(Button(content="Play", activationType="background", arguments="dismiss", pendingUpdate=False))
#toast.addButton(Button(content="Pause", activationType="protocol", arguments='''runas.exe /savecred /user:333danich333@mail.ru "C:/Program Files (x86)/Ubisoft/Ubisoft Game Launcher/Uplay.exe"''', pendingUpdate=False))
#toast.addButton(Button(content="HTTP", activationType="protocol", arguments="https://www.google.com/", pendingUpdate=False))
#toast.setIcon('C:/Users/333da/Desktop/wintoast/0.jpg', crop="circle")
#toast.show()


#getToast("Title", "Message", 'C:/Users/333da/Desktop/wintoast/0.jpg', "circle", "short").show()






class JSElement(object):
    def __init__(self,driver):
        self.driver = driver

    def click(self,class_name=None,id=None,name=None,
                     index=None,describe=None):
        describe = describe
        if class_name:
            class_name = """document.getElementsByClassName("{classname}")
            [{index}].click(); 
            """.format(classname=class_name,index=index)
            d = self.driver.execute_script(class_name)

            return class_name

        if id:

           id =  f"""document.getElementById("{id}").click();"""
           f = self.driver.execute_script(id)
           return name


        if name:
           name =  """document.getElementsByName("{name}")
           [{index}].click();""".format(name=name,index=index)
           return name



    def input(self, id=None,name=None,classname=None, index=None,value=None,describe=None):
        if id:
            id = f"""document.getElementById("{id}").value = "{value}";"""
            self.driver.execute_script(id)
            return id

        describe = describe
        class_name = """document.getElementsByClassName("{classname}")
        ["{index}"].value="{value}"; 
        """.format(classname=classname, index=index,value=value)
        self.driver.execute_script(class_name)

        return class_name

class BrowserObject(object):
    """
    该模类是对javascript 的BOM模型进行封装也就是我们还不能制作一些我们经常看
    到的网页的一些交互，我们需要继续学习BOM和DOM相关知识。
    JavaScript分为 ECMAScript，DOM，BOM。
    BOM（Browser Object Model）是指浏览器对象模型，它使 JavaScript 有能力与
    浏览器进行“对话”。
    """
    #This module class encapsulates the bom model of javascript, that is, we
    # can not make some interactions of the web pages we often see, and we
    # need to continue to learn bom and dom related knowledge. Javascript is
    # divided into ecmascript,dom,bom. Bom (browser object model) refers to
    # the browser object model, which enables javascript to have a "dialogue"
    # with the browser

    def __init__(self,driver):
        self.driver = driver


    def window_height(self,describe=None):
        describe = describe
        win = """var name = window.innerHeight;
                    console.log(name)"""
        win_height = self.driver.execute_script(win)
        return win_height

    def window_open(self,describe=None):
        describe = describe
        win = "window.open();"
        win_Open = self.driver.execute_script(win)
        return win_Open

    def window_close(self,describe=None):
        describe = describe
        win = "window.close();"
        win_close= self.driver.execute_script(win)
        return win_close

    def add_element(self,type=None,tag_name=None,index=None,types=None,
                    values=None,describe=None):
        if tag_name:
            describe = describe
            win = """document.getElementsByTagName
            ("{tagname}")[{indexs}].setAttribute("{types}","{value}");
            """.format(tagname=tag_name,indexs=index,types=types,value=values)
            win_close= self.driver.execute_script(win)

            return win_close




#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2022/8/29


"""
                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|
"""


from faker import Faker

from sveltest.support import LanguageException


class VirtualData:
    def __init__(self,language="zh_CN"):
        # 简体中文：zh_CN
        # 繁体中文：zh_TW
        # 美国英文：en_US
        # 英国英文：en_GB
        # 德文：de_DE
        # 日文：ja_JP
        # 韩文：ko_KR
        # 法文：fr_FR

        if language in [
            "zh_CN","zh_TW","en_US","en_GB","de_DE",
            "ja_JP","ko_KR","fr_FR"
        ]:
            self.get_data = Faker(locale=language)
        else:
            raise LanguageException(msg="不支持的语言类型")



    def country(self,mode=None):
        """

        :param mode: suffix 省市县级 code 国家编码

        :return:
        """
        # 国家
        if mode is None:
            return self.get_data.country()
        elif mode == "suffix":
            return self.city_suffix()
        elif mode == "code":
            return self.get_data.country_code()


    def city_suffix(self):
        """城市的末尾标识"""
        # self.get_data.seed(100)
        return self.get_data.city_suffix()

    def address(self,count=None):
        """地址"""
        if count is None:

            return self.get_data.address()
        else:
            return [self.get_data.address() for x in range(count)]


    def alphabet(self,isupper=False):
        """生成英文字母
        """
        txt = ''
        if isupper is False:
            txt = self.get_data.random_element()
        else:
            while True:
                d = self.get_data.random_letter()
                if d.isupper():
                    txt = d
                    break

        return  txt

    def date(self):
        return self.get_data.date()

    def country_code(self):
        return self.get_data.date()


    def opera_user_agent(self):
        """随机生成Opera的浏览器user_agent信息"""
        return self.get_data.opera()

    def safari_user_agent(self):
        """随机生成Safari的浏览器user_agent信息"""
        return self.get_data.safari()

    def chrome_user_agent(self):
        """随机生成chrome的浏览器user_agent信息"""
        return self.get_data.chrome()

    def firefox_user_agent(self):
        """随机生成firefox的浏览器user_agent信息"""
        return self.get_data.firefox()

    def internet_exploreruser_agent(self):
        """随机生成ie的浏览器user_agent信息"""
        return self.get_data.internet_explorer()

    def linux_platform_token(self):
        """随机Linux信息"""
        return self.get_data.linux_platform_token()

    def user_agent(self):
        """随机user_agent信息"""
        return self.get_data.user_agent()





# ● fake.city_suffix()：市，县
# ● fake.country()：国家
# ● fake.country_code()：国家编码
# ● fake.district()：区
# ● fake.geo_coordinate()：地理坐标
# ● fake.latitude()：地理坐标(纬度)
# ● fake.longitude()：地理坐标(经度)
# ● fake.postcode()：邮编
# ● fake.province()：省份
# ● fake.address()：详细地址
# ● fake.street_address()：街道地址
# ● fake.street_name()：街道名
# ● fake.street_suffix()：街、路

if __name__ == '__main__':
    x = VirtualData()
    print(x.opera_user_agent())



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2021/9/28


import logging.config

from sveltest.bin.conf import settings


standard_format = '%(levelname)s  %(asctime)s  %(threadName)s:%(thread)d_task_id:%(name)s_%(filename)s:%(lineno)d' \
                  ' msg:%(message)s'  # 其中name为getlogger指定的名字

simple_format = '%(levelname)s  %(asctime)s %(lineno)d' \
                ' msg:%(message)s'

id_simple_format = '[%(levelname)s][%(asctime)s] %(message)s'

logging_set_ = getattr(settings, "LOG_FILE_PATH")


LOGGING_CONFIG = {
    'version': 1,

    'disable_existing_loggers': False,
    # 定义日志 格式化的 工具
    'formatters': {
        'standard': {
            'format': standard_format
        },
        'simple': {
            'format': simple_format
        },
        'id_simple': {
            'format': id_simple_format
        },
    },
    # 过滤器
    'filters': {},
    'handlers': {
        # 打印到终端的日志
        'stream': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',  # 打印到屏幕
            'formatter': 'standard'
        },
    },
    # logger实例
    'loggers': {
        # 默认的logger应用如下配置
        '_debug': {
            'handlers': ['stream'],
            'level': 'DEBUG',
            'propagate': True,
        },
    },
}



class Log4J:
    """
    log日志封装
    Attributes:
    """
    def __init__(self):

        logging.config.dictConfig(LOGGING_CONFIG)
        self.logger = logging.getLogger("_debug")

    def info(self,msg):
        """用于打印出info级别的日志
        Args:
            msg:日志描述
        Returns:
            example:
        Raises:
        """
        self.logger.info(msg=msg)

    def debug(self,msg):
        """用于打印出debug级别的日志
        Args:
            msg:日志描述
        Returns:
            example:
        Raises:
        """
        self.logger.debug(msg=msg)

    def error(self,msg):
        """用于打印出error级别的日志
        Args:
            msg:日志描述
        Returns:
            example:
        Raises:
        """
        self.logger.error(msg=msg)

    def warning(self,msg):
        """用于打印出 warning 级别的日志
        Args:
            msg:日志描述
        Returns:
            example:
        Raises:
        """
        self.logger.warning(msg=msg)







#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2022/10/21
import importlib




class ImportString:
    """
    使用字符串进行动态导入模块
    """
    def import_string(self,dotted_path):
        """
        Import a dotted module path and return the attribute/class designated by the
        last name in the path. Raise ImportError if the import failed.
        """
        try:
            module_path, class_name = dotted_path.rsplit('.', 1)

        except ValueError as err:
            raise ImportError("%s doesn't look like a module path" % dotted_path) from err



        module = importlib.import_module(module_path)


        try:
            return getattr(module, class_name)
        except AttributeError as err:
            raise ImportError('Module "%s" does not define a "%s" attribute/class' % (
                module_path, class_name)
                              ) from err


importString = ImportString()








#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2021/11/8

"""
                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|
"""
import argparse
import os
import subprocess

from gooey import Gooey, GooeyParser, PrefixTokenizers


@Gooey(
    program_name="fastTest-CLI GUI TOOL",
    language="chinese",
    image_dir=r"F:\app\1",
    # tabbed_groups=True,
    advanced=True,
    sidebar_title="命令",
    show_sidebar=True, navigation=["Tabbed"],
    menu=[
        {'name': '关于', 'items': [{
            'type': 'AboutDialog',
            'menuTitle': '关于',
            'name': 'fastTest-CLI GUI TOOLS',
            'description': '                  fastTest 框架\n给你带来快速搭建测试项目的极速体验',
            'version': '1.0.0',
            'copyright': '2021',
            'website': 'https://github.com/chriskiehl/Gooey',
            'developer': '',
            'license': 'MIT'
        },
        ]}, {
            'name': '帮助',
            'items': [{
                'type': 'Link',
                'menuTitle': '教程文档',
                'url': 'https://www.readthedocs.com/foo'
            }]
        }],

)
# @Gooey(menu=[
# 	{'name': '文件', 'items': [{
#                 'type': 'AboutDialog',
#                 'menuTitle': 'About',
#                 'name': 'Gooey Layout Demo',
#                 'description': 'An example of Gooey\'s layout flexibility',
#                 'version': '1.2.1',
#                 'copyright': '2018',
#                 'website': 'https://github.com/chriskiehl/Gooey',
#                 'developer': 'http://chriskiehl.com/',
#                 'license': 'MIT'
#             },
# 	]},
#              {'name': 'Tools', 'items': []},
#              {'name': 'Help', 'items': []}],
# 	   language="chinese"
# 	   )

def main():
    parser = GooeyParser(description="fastTest CLI GUI")

    # 子命令
    # parser.add_argument("--create",help='创建工程', dest='create',)
    subs = parser.add_subparsers(help='创建工程', dest='create')
    # add_subparsers(help='commands', dest='command')
    create_parser = subs.add_parser(
        'create', help='创建WEB UI自动化测试工程项目')

    create_parser.add_argument('-type',
                               widget='Dropdown',
                               help='创建的项目类型(ui/api/qt)',
                               type=str,
                               metavar='type',
                               choices=["ui", "api", "app", "qt"],
                               default="ui"
                               )
    create_parser.add_argument('file_path', widget='DirChooser',
                               help='选择需要的目录', type=str,
							   gooey_options={
								   # 输入验证
								   'validator': {
									   # 表单验证规则
									   'test': 'len(user_input) > 0',
									   # 验证描述信息
									   'message': '该选项不能为空'
								   }
							   })

    # 	curl_parser.add_argument('filepath', widget='DirChooser',
    # 						help='选择需要的目录',type=argparse.FileType())
    create_parser.add_argument('project_name', widget='TextField',
                               help='工程项目名称')
    # parser.add_argument('filepath', widget="DirChooser")      # 文件选择框
    # parser.add_argument('val', widget="TextField")      # 文件选择框
    # parser.add_argument('Date', widget="DateChooser")          # 日期选择框

    # ==================================================================
    # add_subparsers(help='commands', dest='command')
    docs_parser = subs.add_parser(
        'open_docs', help='浏览器打开 fastTest 官方教程文档及官方网站')
    docs_parser.add_argument('-doc', action="store_true",
    						 help='直接点击开始即可',default=True)
    # curl_parser.add_argument('val', widget='TextField',
    # 						 help='Option one')

    #===================================================================



    webservice_parser = subs.add_parser(
        'webservice', help='web 服务')
    group3 = webservice_parser.add_argument_group('创建服务类型')
    group3.add_argument('type',
                               widget='Dropdown',
                               help='创建服务类型(接口服务则需要填写分组为接口服务的参数、本地服务器则填写文件服务器分组的配置信息)',
                               type=str,
                               metavar='类型',
                               choices=["接口服务", "本地服务器"],
                               default="接口服务"
                               )

    group1 = webservice_parser.add_argument_group('接口服务')
    # group1.add_argument('--opt1', action='store_true',
    #                     help='Option one')


    # stuff = group1.add_mutually_exclusive_group(
    #     required=True,
    #     gooey_options={
    #         'initial_selection': 0
    #     }
    # )
    group1.add_argument('host', widget="TextField",
    						 help='主机ip',default="0.0.0.0",type=str)

    group1.add_argument('-port', widget="TextField",
    						 help='端口',default="8100",type=str)

    group2 = webservice_parser.add_argument_group('文件服务器')
    group2.add_argument('-H', widget="TextField",
                        help='主机ip', default="0.0.0.0", type=str)
    group2.add_argument('-p', widget="TextField",
    						 help='端口',default="8160",type=str)
    group2.add_argument('-root_dir', widget='DirChooser',

                               help='选择一个目录作为服务器的根目录', type=str,
                               gooey_options={
                                   # 输入验证
                                   'validator': {
                                       # 表单验证规则
                                       'test': 'len(user_input) > 0',
                                       # 验证描述信息
                                       'message': '该选项不能为空'
                                   }
                               })

    # ===================================================================
    # download
    download_parser = subs.add_parser(
        'download', help='下载工具'
    )
    group2 = download_parser.add_argument_group('第三方依赖包下载')
    # group2.add_argument('type',
    #                            widget='Listbox',
    #                            help='选择需要下载的Python(第三方包)依赖',
    #                            type=str,
    #                            metavar='类型',
    #                            choices=["接口服务", "本地服务器"],
    #                            default="接口服务",
    #                              nargs="*"
    #                            )

    group2.add_argument(
        "-P",
        "--package",
        metavar='第三方库',
        help='选择需要下载的Python(第三方包)依赖',
        choices=["Django","Selenium2","Selenium3","redis","tomorrow"],
        widget='FilterableDropdown',
        gooey_options={
            'label_color': (255, 100, 100),
            'placeholder': '可以进行输入关键字进行搜索对应的包',
            'search_strategy': {
                'type': 'PrefixFilter',
                'choice_tokenizer': PrefixTokenizers.WORDS,
                'input_tokenizer': PrefixTokenizers.REGEX('\s'),
                'ignore_case': True,
                'operator': 'AND',
                'index_suffix': False  # set to True to enable substring searching!
            }
        })

    group = download_parser.add_argument_group('选择下载源')
    group.add_argument('type',
                               widget='Dropdown',
                               help='选择使用pip 下载的镜像源',
                               type=str,
                               metavar='类型',
                               choices=[
                                   "默认源",
                                   "清华大学",
                                   "阿里云",
                                   "中国科技大学",
                                   "豆瓣(douban)",
                                   "中国科技大学",
                                   "华中理工大学",
                                   "山东理工大学",

                               ],
                               default="默认源"
                               )


    # 阿里云 http://mirrors.aliyun.com/pypi/simple/
    # 中国科技大学 https://pypi.mirrors.ustc.edu.cn/simple/
    # 豆瓣(douban) http://pypi.douban.com/simple/
    # 清华大学 https://pypi.tuna.tsinghua.edu.cn/simple/
    # 中国科学技术大学 http://pypi.mirrors.ustc.edu.cn/simple/

    # 清华：https://pypi.tuna.tsinghua.edu.cn/simple
    #
    # 阿里云：http://mirrors.aliyun.com/pypi/simple/
    #
    # 中国科技大学 https://pypi.mirrors.ustc.edu.cn/simple/
    #
    # 华中理工大学：http://pypi.hustunique.com/
    #
    # 山东理工大学：http://pypi.sdutlinux.org/
    #
    # 豆瓣：http://pypi.douban.com/simple/
    # pip
    # install - i
    # https: // pypi.tuna.tsinghua.edu.cn / simple(要安装的包)

    # ===================================================================

    #adb shell
    adb_shell_parser = subs.add_parser(
        'adb_shell', help='adb 操作'
    )
    group1 = adb_shell_parser.add_argument_group('设备连接')
    group1.add_argument('-H', widget="TextField",
                        help='设备ip', type=str)
    group1.add_argument('-P', widget="TextField",
                        help='设备端口', type=str)


    # 接收界面传递的参数
    args = parser.parse_args()
    # print(args)
    # print(args.create)
    if args:
        if args.create == 1:
            print("已启动浏览器且已经为你打开了 fastTest 官方教程文档及官方网站")

        if args.create == "download" and args.package is not None:
            # print(args.package)
            os.system(f"pip3 install {args.package}")


        if args.create == "adb_shell":

            if args.H and args.P:
                HOST = f"{args.H}:{args.P}"
                os.system(f"adb connect {HOST}")
                print(HOST)



            # print(subprocess.Popen(k_shell, stdout=subprocess.PIPE).communicate()[0])






# if os.path.exists(filepath):
# 	os.mkdir(filepath + "\\%s" % val)
#
# else:
# 	print("不是文件夹")

# print(xt,name)
# os.mkdir()


# if args:
# 	os.system("start www.baidu.com")	print(1)


if __name__ == '__main__':
    main()




#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2022/10/9


 # b"bin/tools/pdfImg.exe"
PDFIMGPATH = b'YmluL3Rvb2xzL3BkZkltZy5leGU='
# '{base_path} -r {idp} -png "{pdf_file}" "{img_path}\{file_prefix}"'
PDFIMGPATH_SHELL = b'e2Jhc2VfcGF0aH0gLXIge2lkcH0gLXBuZyAie3BkZl9maWxlfSIgIntpbWdfcGF0aH1ce2ZpbGVfcHJlZml4fSI='

# '{base_path} -f {start_page} -l {end_page} -r {idp} -png "{pdf_file}" "{img_path}\{file_prefix}"'
PDFIMG_RANGE = b'e2Jhc2VfcGF0aH0gLWYge3N0YXJ0X3BhZ2V9IC1sIHtlbmRfcGFnZX0gLXIge2lkcH0gLXBuZyAie3BkZl9maWxlfSIgIntpbWdfcGF0aH1ce2ZpbGVfcHJlZml4fSI='





#!/usr/bin/env python
#-*- coding:utf-8 -*-

from sveltest.support.logger_v2 import log_v2
from selenium import webdriver
from selenium.webdriver.chrome.options import Options


def head():
    chrome_options = Options()
    chrome_options.add_argument('--headless')
    chrome_options.add_argument('--disable-gpu')
    chrome_options.add_argument(
        'User-Agent="Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/97.0.4692.99 Safari/537.36')
    return chrome_options


def get_browser_version(driver="chrome",options=head()):



    if driver == "geckodriver" or driver == "Firefox" or driver == "hf":
        log_v2.info("正在获取本地安装 Firefox 浏览器的版本号...")
        driver = webdriver.Firefox(options=options)

    else:
        log_v2.info("正在获取本地安装 Chrome 浏览器的版本号...")
        driver = webdriver.Chrome(options=options)

    info = driver.capabilities.get('chrome').get("chromedriverVersion").split(' ')
    log_v2.info("版本号获取成功！版本为："+info[0])
    driver.quit()
    return info





#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""

                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|


"""
import base64
import io
import os
import re
import shutil
import sys
import zipfile

from typing import Optional,List

# v 1.0
# guanfl
# 2021.5.28
# :TODO 已完成 待单元测试
from sveltest.bin.conf.license import PDFIMGPATH, PDFIMGPATH_SHELL, PDFIMG_RANGE


class StFile:
    def __init__(self):
        pass


    def get_path_list(self,path:Optional[str]) -> List:
        """
        指定目录下的所有文件路径
        :param path:
        :return:
        """

        assert os.path.isdir(path), '%s not exist.' % path
        ret = []
        for root, dirs, files in os.walk(path):
            for filespath in files:
                ret.append(os.path.join(root, filespath))
        return ret

    def copy_all(self,src_path:Optional[str], target_path:Optional[str]) -> Optional[bool]:
        """

        :param src_path:
        :param target_path:
        :return:
        """

        if os.path.isdir(src_path) and os.path.isdir(target_path):
            filelist_src = os.listdir(src_path)
            for file in filelist_src:
                path = os.path.join(os.path.abspath(src_path), file)
                if os.path.isdir(path):
                    path1 = os.path.join(os.path.abspath(target_path), file)
                    if not os.path.exists(path1):
                        os.mkdir(path1)
                    self.copy_all(path, path1)
                else:
                    with open(path, 'rb') as read_stream:
                        contents = read_stream.read()
                        path1 = os.path.join(target_path, file)
                        with open(path1, 'wb') as write_stream:
                            write_stream.write(contents)
            return True
        else:
            return False

    def dir_copy(self,src_path:Optional[str], target_path:Optional[str]):
        """copy
        all
        files
        of
        src_path
        to
        target_path"""
        # 将某文件夹下所有文件复制至指定文件夹内，但不复制该文件夹的结构
        file_count = 0
        source_path = os.path.abspath(src_path)
        target_path = os.path.abspath(target_path)
        if not os.path.exists(target_path):
            os.makedirs(target_path)
        if os.path.exists(source_path):
            for root, dirs, files in os.walk(source_path):
                for file in files:
                    src_file = os.path.join(root, file)
                    shutil.copy(src_file, target_path)
                    file_count += 1
                    print(src_file)
        return int(file_count)

    def isdir(self,path:Optional[str]):
        """

        :param path:
        :return:
        """
        p = os.path.splitext(path)
        if p[1]:
            return False
        else:
            return True

    def isfile(self,path:Optional[str]):
        """

        :param path:
        :return:
        """
        p = os.path.splitext(path)
        if p[-1]:
            if not os.listdir(path):
                return False
            else:
                return True
        else:
            return False


    def is_listdir(self,path:Optional[str]):
        """

        :param path:
        :return:
        """
        file_list = os.listdir(path)
        file = []
        dir = []
        other_files = []
        listfile = {
            "file": file,
            "dir": dir,
            "other_files": other_files
        }
        for i in file_list:
            p = os.path.splitext(i)
            if p[-1]:
                if p[-1] == ".zip" or p[-1] == ".rar":
                    other_files.append(i)
                    print("{} 目录下 {} 压缩文件".format(path, i))
                else:
                    file.append(i)
                    print("{} 目录下 {} 是文件".format(path, i))
            else:
                dir.append(i)
                print("{} 目录下 {} 是文件夹".format(path, i))
        return listfile

    def del_type_file(self,path:Optional[str], filename:Optional[str]):
        """

        :param path:
        :param filename:
        :return:
        """
        pathfile = os.listdir(path)
        for files in pathfile:
            join_file_path = os.path.join(path, files)
            where = os.path.splitext(files)

            if where[-1] == filename:

                os.remove(join_file_path)
                print("已删除{}".format(files))



    def del_all_file(self,path):
        """

        :param path:
        :return:
        """
        pathfile = os.listdir(path)
        for files in pathfile:
            join_file_path = os.path.join(path, files)
            if os.path.isfile(join_file_path):
                os.remove(join_file_path)
                print("已删除{}".format(files))
            else:
                try:
                    self.del_all_file(join_file_path)
                    os.rmdir(join_file_path)
                    print("已删除{}".format(files))
                except:
                    self.del_all_file(join_file_path)
                print("已删除{}".format(files))

    def del_current_file(self,path):
        """

        :param path:
        :return:
        """
        pathfile = os.listdir(path)
        for files in pathfile:
            join_file_path = os.path.join(path, files)
            if os.path.isfile(join_file_path) == True:
                os.remove(join_file_path)
                print("已删除{}".format(files))


class ZipFile:
    """"""
    def __init__(self):
        self.pathFile = None
        self._filer = []

    def all_path(self, path):
        """

        :param path:
        :return:
        """
        # 递归获取目录下所有文件
        join = os.listdir(path)

        for d in join:
            die = os.path.join(path, d)
            try:
                self.all_path(die)
                self._filer.append(die)
            except:
                self._filer.append(die)
        return self._filer

    def zip_file(self,path, zipfile_name):
        """
        Args:
            path:传入一个需要压缩的文件夹路径
            zipfile_name:压缩包的存放路径
        return:
            传出的是一个压缩包的存放路径
        """
        # 压缩zip文件 path压缩的文件夹路径 压缩文件路径+.zip
        path = path
        join = os.listdir(path)
        join_path = []

        for i in join:
            public = os.path.join(path, i)
            join_path.append(public)

        f = zipfile.ZipFile(zipfile_name, 'w', zipfile.ZIP_DEFLATED)

        for s in join_path:
            jie = s.replace("\\", "/").split('/')
            index = jie[-1]
            f.write(s.replace("\\", "/"), index)

        f.close()

        return zipfile_name

    def all_zip(self, zipfile_name, path):
        """
        打包所有
        :param zipfile_name:
        :param path:
        :return:
        """
        _path = self.all_path(path)
        f = zipfile.ZipFile(zipfile_name, 'w', zipfile.ZIP_DEFLATED)
        for adb in _path:
            f.write(adb)

        f.close()
        return _path


    def extract(self,filename):
        """
        进行解压
        :param filename:
        :return:
        """
        z = zipfile.ZipFile(filename)
        for f in z.namelist():
            # get directory name from file
            dirname = os.path.splitext(f)[0]
            # create new directory
            os.mkdir(dirname)
            # read inner zip file into bytes buffer
            content = io.BytesIO(z.read(f))
            zip_file = zipfile.ZipFile(content)
            for i in zip_file.namelist():
                zip_file.extract(i, dirname)

# 是否包含中文字符
def is_chinese(string:Optional[str]) -> bool:
    """
    检查整个字符串是否包含中文
    :param string: 需要检查的字符串
    :return: bool
    """
    for ch in string:
        if u'\u4e00' <= ch <= u'\u9fff':
            return True

    return False


class FileHandle:
    """
    文件操作
    """


    def link(self,path,oj):
        """
         用于创建硬连接的源地址
         用于创建硬连接的目标地址
        :param path:
        :param oj:
        :return:
        """
        try:
            os.link(path,oj)
            return 0
        except:
            return 1

    def isfile(self,file):
        """
        是否存在该文件
        :return:
        """
        return os.path.exists(file)



# v1.2.2
# 20210723
from PIL import Image

import subprocess


class LicenseBaseDecode:
    """

    """
    PDFIMG_EXE_PATH = base64.b64decode(PDFIMGPATH).decode()
    # print(PDFIMG_EXE_PATH)



code = LicenseBaseDecode


class ToolsBase:
    BASE_DIR = os.path.dirname(os.path.dirname((os.path.abspath(__file__))))

    PDFIMG = os.path.join(BASE_DIR, code.PDFIMG_EXE_PATH).replace('\\', '/')

    PDFIMG_SHELL = base64.b64decode(PDFIMGPATH_SHELL).decode()
    PDFIMG_RANGE_IMG = base64.b64decode(PDFIMG_RANGE).decode()



base_commandline = ToolsBase
class PDFObject:

    def __init__(self,path=None):
        """
          -f <int>                 : 要转换的前面第几页
          -l <int>                 : 要转换的最后一页
          -o                       : 只打印奇数页
          -e                       : 只打印偶数页
          -singlefile              : 只写第一页，不要加数字
          -r <fp>                  : resolution, in DPI (default is 150) 分辨率
          -rx <fp>                 : X分辨率，DPI(默认为150)
          -ry <fp>                 : Y resolution, in DPI (default is 150)
          -scale-to <int>          : 缩放每一页以适应从比例到*比例到像素的框
          -scale-to-x <int>        : scales each page horizontally to fit in scale-to-x pixels
          -scale-to-y <int>        : scales each page vertically to fit in scale-to-y pixels
          -x <int>                 : x-coordinate of the crop area top left corner
          -y <int>                 : y-coordinate of the crop area top left corner
          -W <int>                 : 裁剪区域的宽度(以像素为单位)(默认为0)
          -H <int>                 : height of crop area in pixels (default is 0)
          -sz <int>                : 裁剪正方形的大小(以像素为单位)(集合W和H)
          -cropbox                 : 使用裁剪框而不是媒体框
          -mono                    : generate a monochrome PBM file
          -gray                    : generate a grayscale PGM file
          -png                     : generate a PNG file
          -jpeg                    : generate a JPEG file
          -jpegopt <string>        : jpeg options, with format <opt1>=<val1>[,<optN>=<valN>]*
          -tiff                    : generate a TIFF file
          -tiffcompression <string>: set TIFF compression: none, packbits, jpeg, lzw, deflate
          -freetype <string>       : enable FreeType font rasterizer: yes, no
          -thinlinemode <string>   : set thin line mode: none, solid, shape. Default: none
          -aa <string>             : enable font anti-aliasing: yes, no
          -aaVector <string>       : enable vector anti-aliasing: yes, no
          -opw <string>            : owner password (for encrypted files)
          -upw <string>            : user password (for encrypted files)
          -q                       : don't print any messages or errors
          -v                       : print copyright and version info
          -h                       : print usage information
          -help                    : print usage information
          --help                   : print usage information
          -?                       : print usage information
        """
        self.path = path
        # 获取当前文件的父级目录名称




    def _rea(self,path, pdf_name):
        """
        目前仅支持 jpg、png、jpeg格式图片
        :param path:
        :param pdf_name:
        :return:
        """
        file_list = os.listdir(path)
        pic_name = []
        im_list = []
        for x in file_list:
            if "jpg" in x or 'png' in x or 'jpeg' in x:
                pic_name.append(x)

            pic_name.sort()
            new_pic = []

            for x in pic_name:
                if "jpg" in x:
                    new_pic.append(x)

            for x in pic_name:
                if "png" in x:
                    new_pic.append(x)


            im1 = Image.open(os.path.join(path, new_pic[0]))

            new_pic.pop(0)
        for i in new_pic:

            img = Image.open(os.path.join(path, i))
            if img.mode == "RGBA":
                img = img.convert('RGB')

                im_list.append(img)
            else:
                im_list.append(img)

        im1.save(pdf_name, "PDF", resolution=100.0, save_all=True, append_images=im_list)

    def save(self,filepath):
        """

        """
        if ".pdf" in filepath:
            self._rea(self.path, pdf_name=filepath)
            print("pdf已生成完成")
        else:
            self._rea(self.path, pdf_name="{}.pdf".format(filepath))
            print("pdf已生成完成")


    def png(self, pdf_file:Optional[str], img_path:Optional[str],
            file_prefix:Optional[str]="sveltest",dpi=150,
            range_out:Optional[List]=None
            ) -> Optional[object]:
        """

        """

        if is_chinese(img_path):
            raise Exception("图片输出的路径不能包含中文")

        if range_out:
            self._range_png(pdf_file=pdf_file,img_path=img_path,file_prefix=file_prefix,dpi=dpi,
                            first_page=range_out[0],last_page=range_out[-1])

        return subprocess.Popen(
            base_commandline.PDFIMG_SHELL.format(
                base_path=base_commandline.PDFIMG, pdf_file=pdf_file, img_path=img_path,
                file_prefix=file_prefix,idp=dpi
            ),
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        ).communicate()[0]

    def _range_png(self, pdf_file:Optional[str], img_path:Optional[str],
            file_prefix:Optional[str]="sveltest",dpi=150,
            first_page=None,last_page=None,
            ) -> Optional[object]:


        return subprocess.Popen(
            base_commandline.PDFIMG_RANGE_IMG.format(
                base_path=base_commandline.PDFIMG, pdf_file=pdf_file, img_path=img_path,
                file_prefix=file_prefix,idp=dpi,start_page=1,end_page=1
            ),
            stdout=subprocess.PIPE,
            stderr=subprocess.STDOUT
        ).communicate()[0]






    def jpeg(self, pdf_file, path):
        """

        """
        subprocess.Popen('"%s" -jpeg "%s" "%s"\image' % (self.PDFTOPPMPATH, pdf_file, path)).communicate()[0]



class TXTObject:

    def __init__(self):
        """
        """
        # 获取当前文件的父级目录名称
        self.BASE_DIR = os.path.dirname(os.path.dirname((os.path.abspath(__file__))))
        self.PDFTOTEXT = os.path.join(self.BASE_DIR, 'default/conf/poppler/bin/pdftotext.exe').replace('\\', '/')

    def txt(self,pdf_file,path,filename=None):
        """
        用法：pdftotext [options] <PDF-file> [<text-file>]
          -f <int> : 要转换的第一页
          -l <int> : 要转换的最后一页
          -r <fp> : 分辨率，以 DPI 为单位（默认为 72）
          -x <int> : 裁剪区域左上角的 x 坐标
          -y <int> : 裁剪区域左上角的 y 坐标
          -W <int> ：以像素为单位的裁剪区域宽度（默认为 0）
          -H <int> : 裁剪区域的高度（以像素为单位）（默认为 0）
          -layout : 保持原来的物理布局
          -fixed <fp> ：假设固定间距（或表格）文本
          -raw ：按内容流顺序保留字符串
          -htmlmeta : 生成一个简单的 HTML 文件，包括元信息
          -enc <string> : 输出文本编码名称
          -listenc : 列出可用的编码
          -eol <string> : 输出行尾约定（unix、dos 或 mac）
          -nopgbrk : 不要在页面之间插入分页符
          -bbox ：将每个单词和页面大小的边界框输出到 html。设置 -htmlmeta
          -bbox-layout ：类似于 -bbox 但具有额外的布局边界框数据。设置 -htmlmeta
          -opw <string> ：所有者密码（用于加密文件）
          -upw <string> : 用户密码（用于加密文件）
          -q : 不打印任何消息或错误
          -v : 打印版权和版本信息
          -h : 打印使用信息
          -help : 打印使用信息
          --help : 打印使用信息
          -？: 打印使用信息
        :param pdf_file:
        :param path:
        :return:
        """


        # -raw 保留字符顺序流
        # layout 按照布局进行输出文本内容
        # htmlmeta 生成简单的html

        p = subprocess.Popen('"%s" -htmlmeta1 "%s" "%s"\math_hw.txt' % (self.PDFTOTEXT, pdf_file, path)).communicate()[0]


class ReNumber:
    def __init__(self):
        self.val = None

    def number(self,str):
        """匹配数字"""
        pattern = re.compile(r'^[0-9]*$')
        if pattern.findall(str):
            self.val = pattern.findall(str)
            return  True
        else:
            return False

    def numbers(self,str,n):
        """n 位数"""
        pattern = re.compile(r'^\d{%s}$'%n)
        if pattern.findall(str):
            self.val = pattern.findall(str)
            return True
        else:
            return False

    def least_number(self,str,n):
        """至少多少位数字"""
        pattern = re.compile(r'^\d{%s,}$'%n)
        if pattern.findall(str):
            self.val = pattern.findall(str)
            return True
        else:
            return False


    def border_number(self,str,min,max):
        """边界值min、max"""
        pattern = re.compile(r'^\d{%s,%s}$'%(min,max))
        if pattern.findall(str):
            self.val = pattern.findall(str)
            return True
        else:
            return False



#!/usr/bin/env python
# -*- coding:utf-8 -*-
"""


                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|



"""

import unittest
import warnings
import sys
import time
from concurrent.futures.thread import ThreadPoolExecutor
from typing import (Optional, List, Union, Dict, Any, NoReturn, Tuple)
from sveltest.bin.conf import settings
from functools import wraps
from unittest import TestResult
from sveltest.support.logger_v2 import log_v2
from .components.toast import test_toast
from .loader import svelteTestLoader
from rich.console import Console, OverflowMethod
from io import StringIO
from unittest.signals import registerResult



get_case = getattr(settings, "CASE_SUITE_PATH")

console = Console()

def failfast(method):
    @wraps(method)
    def inner(self, *args, **kw):
        if getattr(self, 'failfast', False):
            self.stop()
        return method(self, *args, **kw)

    return inner

class OutputRedirector(object):
    """ 包装器重定向stdout或stderr """

    def __init__(self, fp):
        self.fp = fp

    def write(self, s):
        self.fp.write(s)

    def writelines(self, lines):
        self.fp.writelines(lines)

    def flush(self):
        self.fp.flush()

STDOUT_LINE = '\nStdout:\n%s'
STDERR_LINE = '\nStderr:\n%s'

stdout_redirector = OutputRedirector(sys.stdout)

stderr_redirector = OutputRedirector(sys.stderr)

class CommonConfig:
    """"""
    pass

TEST_CASE_SET_OUT = []

class SvelteResult(TestResult):
    """

    """

    def stopTest(self, test: Optional[object]):
        """

        :param test:
        :return:
        """
        """Called when the given test has been run"""

        setattr(CommonConfig, "run", {
            "count": self.testsRun,
            "errors": len(self.errors),
            "failures": len(self.failures),
            "skipped": len(self.skipped),
        })


class SvelteTestResult(SvelteResult):
    """

    """

    def __init__(self, stream: Optional[Any],
                 descriptions: Optional[str],
                 verbosity: Optional[int],
                 module: Optional[str]):
        """

        :param stream:
        :param descriptions:
        :param verbosity:
        :param module:
        """
        super(SvelteTestResult, self).__init__()
        self.stdout = None
        self.stderr = None
        self.stream = stream
        self.verbosity = verbosity
        self.descriptions = descriptions
        self.module = module
        self.outputBuffer = StringIO()
        self.result = []



    def startTest(self, test):
        test.imgs = []
        TestResult.startTest(self, test)
        # TestResult.startTest(self, test)
        # 只有一个用于stdout和stderr的缓冲区
        self.outputBuffer.seek(0)
        self.outputBuffer.truncate()
        stdout_redirector.fp = self.outputBuffer
        stderr_redirector.fp = self.outputBuffer
        self.stdout0 = sys.stdout
        self.stderr0 = sys.stderr
        sys.stdout = stdout_redirector
        sys.stderr = stderr_redirector

    def complete_output(self):
        """
        """
        if self.stdout0:
            sys.stdout = self.stdout0
            sys.stderr = self.stderr0
            self.stdout0 = None
            self.stderr0 = None
        return self.outputBuffer.getvalue()


    def addSuccess(self, test:Optional[Any],
                   *args:Optional[Tuple],
                   **kwargs:Optional[Dict]
                   ):
        """

        :param test:
        :param args:
        :param kwargs:
        :return:
        """

        add_success = str(self.module).replace('\\', '/').split('/')[-1].rstrip("'>")
        super(SvelteTestResult, self).addSuccess(test)

        output = self.complete_output()

        if self.verbosity > 1:
            context_ = "%s    %s   PASS" % (add_success, test)
            console.print(context_, style="green")
            case_name_tag = ".".join([r for r in context_.split(" ") if r != ""][:2])
            TEST_CASE_SET_OUT.append({case_name_tag:output})

        elif self.verbosity:
            # 成功
            context_ = "%s   OK" % (test)
            console.print(context_, style="green")
            case_name_tag = None



    def addError(self, test:Optional[Any], err:Optional[str]):
        """

        :param test:
        :param err:
        :return:
        """

        add_success = str(self.module).replace('\\', '/').split('/')[-1].rstrip("'>")
        super(SvelteTestResult, self).addError(test, err)

        output = self.complete_output()

        if self.verbosity > 1:
            context_ = "%s    %s   ERROR" % (add_success, test)
            console.print(context_, style="red")
            case_name_tag = ".".join([r for r in context_.split(" ") if r != ""][:2])
            TEST_CASE_SET_OUT.append({case_name_tag: output})
        elif self.verbosity:

            console.print("%s   E" % (test), style="red")

    def addFailure(self, test:Optional[Any],err:Optional[str]):
        """

        :param test:
        :param err:
        :return:
        """



        # 加载断言失败的用例
        add_failure = str(self.module).replace('\\', '/').split('/')[-1].rstrip("'>")
        super(SvelteTestResult, self).addFailure(test, err)
        output = self.complete_output()


        if self.verbosity > 1:
            context_ = "%s    %s   FAIL" % (add_failure, test)

            console.print(context_, style="red")
            case_name_tag = ".".join([r for r in context_.split(" ") if r != ""][:2])
            TEST_CASE_SET_OUT.append({case_name_tag: output})

        elif self.verbosity:
            console.print("%s   F" % (test), style="red")



    def addSkip(self, test: Optional[Any], reason: Optional[str]):
        """

        :param test:
        :param reason:
        :return:
        """
        # overflow_methods: List[OverflowMethod] = ["跳过的用例"]
        # for overflow in overflow_methods:
        # console.rule(title=overflow_methods[0], characters="-", style='yellow')
        add_skip = str(self.module).replace('\\', '/').split('/')[-1].rstrip("'>")
        super(SvelteTestResult, self).addSkip(test, reason)
        context_ = "%s    %s   SKIP" % (add_skip, test)
        if self.verbosity > 1:
            console.print("%s    %s   SKIP" % (add_skip, reason), style="yellow")
            case_name_tag = ".".join([r for r in context_.split(" ") if r != ""][:2])
            TEST_CASE_SET_OUT.append({case_name_tag: output})

        elif self.verbosity:
            console.print("%s   S" % ( reason), style="yellow")

    def printErrors(self, e:Optional[int]):
        """

        :param e:
        :return:
        """
        if self.verbosity:
            self.stream.writeln()

        if e != 0:
            overflow_methods: List[OverflowMethod] = ["铺抓到的异常"]
            # for overflow in overflow_methods:
            console.rule(title=overflow_methods[0], characters="-", style='bold blue')
            self.printErrorList('ERROR', self.errors)
            self.printErrorList('FAIL', self.failures)

    def printErrorList(self, flavour:Optional[str], errors:Optional[Any]):
        """

        :param flavour:
        :param errors:
        :return:
        """
        # 异常用例结果输出

        try:
            for test, err in errors:
                err_f = err
                err_case = err_f.split("\n")[1].split(" ")[3:]
                _path = str(err_case[0].split('"')[1])

                console.print(">>> " + str(test), style="red")
                console.print(err_f, style="red")
        except:

            for test in errors:

                console.print(">>> " + str(test[-1]), style="red")
                for x in str(test[1]).split("\n"):
                    console.print(x, style="red")


class _WritelnDecorator(object):
    """Used to decorate file-like objects with a handy 'writeln' method"""

    def __init__(self, stream):
        self.stream = stream

    def __getattr__(self, attr):
        if attr in ('stream', '__getstate__'):
            raise AttributeError(attr)
        return getattr(self.stream, attr)

    def writeln(self, arg=None):
        if arg:
            self.write(arg)
        self.write('\n')  # text-mode streams translate to \r\n if needed


class SvelteTextTestRunner(object):
    """以文本形式显示结果的测试运行器类。
        它在运行时打印测试的名称，在运行时打印错误
        发生，并在测试运行结束时汇总结果。
    """

    # resultclass =

    def __init__(self, stream: Optional[object] = None,
                 descriptions: Optional[Union[str, bool]] = True,
                 verbosity: Optional[Union[str, int]] = 1,
                 failfast: Optional[bool] = False,
                 buffer: Optional[bool] = False,
                 warnings: Optional[str] = None,
                 *, tb_locals: Optional[bool] = False,
                 module: Optional[Any] = None):
        """
        构造一个TextTestRunner。
        子类应该接受**kwargs以确保兼容性
        界面的变化。
        :param stream:
        :param descriptions:
        :param verbosity:
        :param failfast:
        :param buffer:
        :param warnings:
        :param tb_locals:
        :param module:
        """

        if stream is None:
            stream = sys.stderr
        self.stream = _WritelnDecorator(stream)
        self.descriptions = descriptions
        self.verbosity = verbosity
        self.failfast = failfast
        self.buffer = buffer
        self.tb_locals = tb_locals
        self.warnings = warnings
        self.module = module
        self.stdout0 = sys.stdout
        self.stderr0 = sys.stderr
        self.outputBuffer = StringIO()

    def run(self, test, max_workers: Optional[int] = 0,toast:Optional[bool]=True):
        """

        :param test:
        :return:
        """


        result = SvelteTestResult(stream=self.stream, descriptions=self.descriptions, verbosity=self.verbosity,
                                  module=self.module, )




        if test.countTestCases() > 0:

            overflow_methods: List[OverflowMethod] = ["用例开始执行"]
            console.rule(title=overflow_methods[0], characters="=", style='bold blue')

            # "联合给定的测试用例或测试套件."

            registerResult(result)
            result.failfast = self.failfast
            result.buffer = self.buffer
            result.tb_locals = self.tb_locals
            with warnings.catch_warnings():
                if self.warnings:

                    warnings.simplefilter(self.warnings)

                    if self.warnings in ['default', 'always']:
                        warnings.filterwarnings('module',
                                                category=DeprecationWarning,
                                                message=r'Please use assert\w+ instead.')
                startTime = time.perf_counter()
                startTestRun = getattr(result, 'startTestRun', None)



                if startTestRun is not None:
                    startTestRun()


                try:
                    if max_workers != 0:
                        with ThreadPoolExecutor(max_workers=max_workers) as ts:
                            for suite in test:
                                # 将套件的执行提交到线程池中
                                ts.submit(suite.run, result=result)
                    else:
                        from io import StringIO
                        import string, sys
                        # stdout = sys.stdout

                        # sys.stdout = file = StringIO()
                        test(result)
                        # sys.stdout = stdout



                finally:
                    stopTestRun = getattr(result, 'stopTestRun', None)
                    if stopTestRun is not None:
                        stopTestRun()

                stopTime = time.perf_counter()


            if self.verbosity > 2:
                overflow_methods: List[OverflowMethod] = ["调试输出"]
                console.rule(title=overflow_methods[0], characters="~", style='bold blue')

                # log_v2.info(TEST_CASE_SET_OUT)

                for p in TEST_CASE_SET_OUT:
                    for e in p.items():
                        console.print(e[0]+"【output】：\n",e[-1]+"\n")




            timeTaken = stopTime - startTime

            get_run_test = getattr(CommonConfig, "run")
            count_test_success = get_run_test["count"] - (
                    get_run_test["errors"] + get_run_test["failures"] + get_run_test["skipped"]
            )

            result.printErrors(get_run_test["errors"] + get_run_test["failures"])

            run = result.testsRun

            overflow_methods: List[OverflowMethod] = ["测试结果汇总"]
            console.rule(title=overflow_methods[0], characters="*", style='bold blue')

            from rich.table import Table
            table = Table(title="执行结果", show_header=True, header_style="bold yellow")
            table.add_column('status', no_wrap=True)
            table.add_column('count', no_wrap=True)

            data = [
                ['PASS', str(count_test_success)],
                ['FAIL', str(get_run_test["failures"])],
                ['SKIP', str(get_run_test["skipped"])],
                ['ERROR', str(get_run_test["errors"])],
                ['COUNT', str(get_run_test["count"])],

            ]
            if toast:
                test_toast(pass_count=count_test_success,fail_count=str(get_run_test["failures"]),
                           skip_count=str(get_run_test["skipped"]),error_count=str(get_run_test["errors"],),
                           count=str(get_run_test["count"])
                           )

            for case in data:
                table.add_row(case[0], case[1])
            console.print(table)

            case_stop = "总共运行了 %d 条测试用例  总共运行了 %.3fs" % ((run, timeTaken))
            overflow_methods: List[OverflowMethod] = [case_stop]
            console.rule(title=overflow_methods[0], characters="=", style='bold blue')


            expectedFails = unexpectedSuccesses = skipped = 0
            try:
                results = map(len, (result.expectedFailures,
                                    result.unexpectedSuccesses,
                                    result.skipped))
            except AttributeError:
                pass
            else:
                expectedFails, unexpectedSuccesses, skipped = results

            infos = []
            if not result.wasSuccessful():

                failed, errored = len(result.failures), len(result.errors)
                if failed:
                    infos.append("failed=%d" % failed)
                if errored:
                    infos.append("errored=%d" % errored)

            if skipped:
                infos.append("skipped=%d" % skipped)
            if expectedFails:
                infos.append("expectedFails=%d" % expectedFails)
            if unexpectedSuccesses:
                infos.append("unexpected successes=%d" % unexpectedSuccesses)

            return result


        else:
            log_v2.warning("没有找到可执行的测试用例")


class SvelteMain(object):
    """

    """

    def __init__(self, run_case_path: Optional[Any] = "__main__",
                 debug: Optional[bool] = True,
                 verbosity: Optional[int] = 1, failfast: Optional[Union[str, bool, int]] = None,
                 title: Optional[str] = "sveltest Test Report", tester: Optional[str] = "Anonymous",
                 description: Optional[str] = "Test case execution",
                 save_last_try: Optional[bool] = False,
                 thread_count: int = 0,
                 toast:Optional[bool]=True
                 ):
        """"""

        self.path = run_case_path
        self.debug = debug
        self.title = title
        self.tester = tester
        self.description = description
        self.verbosity = verbosity
        self.failfast = failfast
        # 测试执行的对象驱动
        # self.exec_driver = exec_driver
        self.runner = unittest.TextTestRunner
        self.report = None
        self.save_last_try = save_last_try
        # 测试用例集
        self.testSuites = []
        self.thread_count = thread_count

        self.module = __import__(self.path)

        if self.path:

            if self.debug is False or getattr(settings, "DEBUG") is False:

                if settings.CASE_SUITE_PATH:
                    case = unittest.defaultTestLoader.discover(start_dir=settings.CASE_SUITE_PATH,
                                                               pattern=settings.TEST_CASE_ENFORCE_RULES)

                else:
                    raise Exception("请在使用manage.py文件进行非DEBUG模式执行测试")


            else:
                case = svelteTestLoader.loadTestsFromModule(self.module)

            self.run(case,toast)

    def run(self, suits: Optional[Any],toast:Optional[bool]=True) -> NoReturn:
        """
        run test case
        """
        if self.debug is True and getattr(settings, "DEBUG") is True:
            # debug 调试模式将启用控制台样式执行器
            testrunner = SvelteTextTestRunner(
                verbosity=self.verbosity,
                failfast=self.failfast,
                module=self.module
                # buffer=self.buffer,
                # warnings=self.warnings,
                # tb_locals=self.tb_locals
            )
            testrunner.run(
                test=suits, max_workers=self.thread_count,toast=toast
            )

        if self.debug is False or getattr(settings, "DEBUG") is False:
            #
            from sveltest.bin.conf.commandline import MainTestSuite
            MainTestSuite().test_suite_base(
                suites=suits,
                test_title_name=self.title, description=self.description, thread_count=self.thread_count,
                save_last_try=True
            )


main = SvelteMain






#!/usr/bin/env python
#-*- coding:utf-8 -*-



"""

                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|



"""


import unittest

from typing import (NoReturn, Optional)
from sveltest.components.network.main import RequestBase


class TestCase(unittest.TestCase):
    """基础类"""


    def __init__(self,methodName='runTest'):
        """

        """
        super(TestCase, self).__init__(methodName=methodName)


    def dispatch(self):

        pass



    def start_class(self) -> NoReturn:
        """"""
        pass

    def end_class(self) -> NoReturn:
        """
        """
        pass

    @classmethod
    def setUpClass(cls) -> NoReturn:
        cls().start_class()

    @classmethod
    def tearDownClass(cls) -> NoReturn :
        cls().end_class()

    def setUp(self) -> NoReturn :
        pass

    def tearDown(self) -> NoReturn :
        pass


    def run(self, result: Optional[unittest.result.TestResult] = ...) \
            -> Optional[unittest.result.TestResult]:

        return super().run(result)


class HttpTestCase(TestCase,RequestBase):
    """
    用于HTTP测试
    """

    def __init__(self,methodName='runTest'):
        """

        """
        super(HttpTestCase, self).__init__(methodName=methodName)
        self.get_auth_class()



    def initizlize_request(self):
        return None


class WebsocketTestCase(TestCase,RequestBase):
    """
    用于websocket测试
    """



class TestCaseSet(HttpTestCase,WebsocketTestCase):
    """
    用于HTTP测试
    """




if __name__ == '__main__':
    HttpTestCase()



#!/usr/bin/env python
#-*- coding:utf-8 -*-


import os
from sveltest.support import Log4J


log4j = Log4J()
var = ""


import random


def getID(index=8):
    """
    用于生成随机字符串
    :param index:
    :return:
    """
    id = ''

    var = '0123456789zxcvbnmasdfghjklqwertyuiopPOIUYTREWQASDFGHJKLMNBVCXZ'

    for i in range(index):
        name = random.choice(var)
        id = name+id

    return id


def is_dir(path):
    """
    判断是否为文件夹
    :param path:
    :return:
    """
    p = os.path.splitext(path)
    if p[0]:
        return True
    else:
        return False


import difflib


# TODO : 文本检查结果
class DiffLibPush:
    def __init__(self):
        pass

    def diff_context(self,a,b):
        """

        :param a:
        :param b:
        :return:
        """

        return difflib.context_diff(a, b)


    def diff_matcher_ratio(self,str1, str2):
        """比较差异的精度比例"""
        return difflib.SequenceMatcher(None, str1, str2).quick_ratio()


    def diff_matcher(self,str1, str2):
        """比较差异的精度比例"""
        return difflib.SequenceMatcher(None, str1, str2).get_opcodes()


    def diff_matcher_format(self,reality, expect):
        """格式化对比
            equal:相同
            replace:被取代
            (tag, i1, i2, j1, j2)，其中tag表示动作，i1表示序列a的开始位置，i2表示序列a的结束位置，j1表示序列b的开始位置，j2表示序列b的结束位置。
            reality:实际 expect 预期
        """
        log4j.info(msg="实际文本内容为: %s,期望文本内容为: %s"%(reality,expect))
        diff_text_replace = ''
        result = []
        _diff =  difflib.SequenceMatcher(None, reality, expect).get_opcodes()
        text_cum = 0 #记录的是哪些存在差异的字符串数
        for tag,i1,i2,j1,j2 in _diff:
            if tag != "equal":
                log4j.info(msg="%s 实际文本结果:[%s]__期望文本结果:[%s][%d:%d]_[%s] obj2:[%s:%s]_[%s]" \
                                    ""%(tag,reality,expect,i1,i2,reality[i1:i2],j1,j2,expect[j1:j2]))
                # diff_text_replace = "%s 实际文本结果:[%s]__期望文本结果:[%s][%d:%d]_[%s] obj2:[%s:%s]_[%s]" \
                #                     ""%(tag,reality,expect,i1,i2,reality[i1:i2],j1,j2,expect[j1:j2])
                if i1 - i2 != 0 or j1 -j2 != 0:
                    result.append(reality[i1:i2]) # 将实际文本中的字符与期望字符中不存在的字符进行保存到result列表中
                    result.append(expect[j1:j2]) # 将期望文本中的字符与实际字符中不存在的字符进行保存到result列表中
                    text_cum += 1


        # text_cum 为0证明文本没有出现差异字符 其他均为有存在差异字符
        if text_cum == 0:
            log4j.info(msg="本次检测识别结果未发现存在差异字符")
            return 0
        else:
            log4j.info(msg="本次检测识别出%s个字符存在差异"%len("".join(result)))
            log4j.info(msg="差异字符为 %s"%"".join(result))
        return result


class ObjectDict(dict):

    def __getattr__(self, key):
        if key not in self:
            return None
        else:
            value = self[key]

            if isinstance(value,dict):

                value = ObjectDict(value)

            return value



#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""


                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|


"""



import os
import importlib
from sveltest.bin.conf import global_settings




class Settings():
    # 初始化项目配置信息
    def __init__(self):
        # 遍历默认的配置文件，并将配置信息添加到，配置对象当中
        for setting in dir(global_settings):
            # 判断是否是大写
            if setting.isupper():
                # 通过反射添加值
                setattr(self, setting, getattr(global_settings, setting))

            # 获取用户的模块字符串
        try:
            mod = os.environ.get('SVELTEST_TEST_SETTINGS_MODULE')

            # importlib可以通过字符串进行导入模块
            module = importlib.import_module(mod)

            # 遍历用户配置文件，并将配置信息添加到对象中，如果重复会覆盖之前系统的配置信息
            for setting in dir(module):

                if setting.isupper():
                    setattr(self, setting, getattr(module, setting))

        except:
            pass


settings = Settings()




#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2022/11/1
import os
import shelve
from pathlib import Path
from typing import (Optional,Union,Any)


class ShelveBase:


    def __init__(self,filename:Optional[str]=None,flag:str="c",protocol:Optional[Union[bool,int]]=None,
                 writeback:Optional[bool]=False):
        """

        """

        self.BASE_DIR = Path(__file__).resolve().parent

        if filename:
            self.__db = shelve.open(filename=filename, flag=flag, protocol=protocol,
                                    writeback=writeback)
        else:
            self.__db = shelve.open(filename=os.path.join(self.BASE_DIR, 'db_dir/cache').replace("\\", "/"), flag=flag,
                                    protocol=protocol, writeback=writeback)


    def add(self,k:Optional[str],v:Optional[Any]) -> bool:
        """

        """
        try:
            self.__db[k] = v
            return True
        except:
            return False

    def update(self,k:Optional[str],v:Optional[Any]) -> bool:
        pass


    def exists(self,k:Optional[str]) -> bool:
        if k in self.__db:
            return True
        else:
            return False

    def get(self,k:Optional[str]):
        try:
            return self.__db[k]
        except:
            return None

    def delete(self,k:Optional[str]) -> bool:
        try:
            del self.__db[k]
            return True
        except:
            return False

    def quit(self):
        self.__db.close()

    def clear(self,path:Optional[str]=None):
        """

        """
        for x,r,d in os.walk(path if path else self.BASE_DIR):
            for de in d:
                if str(de).endswith(".py"):
                    continue
                else:
                    if os.path.isfile(os.path.join(x,de).replace("\\","/")):
                        os.remove(os.path.join(x,de).replace("\\","/"))




if __name__ == '__main__':
    data_cache = ShelveBase()
    # data_cache.add(k="sveltest",v={"name":"自动化测试","type":0})
    # data_cache.add(k="自动化测试",v="sveltest框架")
    # data_cache.add(k="obj",v=data_cache)
    # # 添加一个key 并且给key赋值,如果存在则修改它的值
    # s.add("小明","1")
    # s.add("小明","2")
    # # 查看一个key是否存在于缓存(数据库)
    # print(s.exists("小明"))
    # # 获取一个key的值
    data_cache.clear()
    data_cache.clear(path="cache")

    # # 删除一个存在的key
    # s.delete("小明")
    # # 清理数据库缓存文件
    # # s.clear()
    # # 关闭并退出数据库
    # s.quit()
    #





#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2022/10/31

from typing import Optional,Union,Dict



"""
基于内存缓存
"""
import time


class Value:
    def __init__(self, value, put_time, expired):
        """
        缓存值对象

        :param value: 具体的值
        :param put_time: 放入缓存的时间
        :param expired: 缓存失效时间
        """
        self.value = value
        self.put_time = put_time
        self.expired = expired



class CacheBase:
    exec_ = 0

    def __init__(self):
        self.__cache = {}
        self.__cache_all = {}

    def set(self, k, v, expired):
        """
        将值放入缓存中

        :param k: 缓存的 key
        :param v: 缓存值
        :param expired: 缓存失效时间，单位秒(s)
        """
        current_timestamp = int(time.time())  # 获取当前时间戳 10 位 秒级
        value = Value(v, current_timestamp, expired)
        self.__cache[k] = value
        self.__cache_all[k] = v


    def check(self, k):
        """
        检查缓存是否可用

        :param k: 缓存 key
        :return: True or False
        """
        current_timestamp = int(time.time())
        value = self.__cache.get(k, None)

        if value is None:
            return False
        differ = current_timestamp - value.put_time

        if differ > value.expired:
            del self.__cache[k]
            del self.__cache_all[k]
            return False
        return True

    def get(self, k:Optional[str]=None):
        """
        通过缓存key获取值

        :param k: key
        :return: value
        """
        if k:
            if self.check(k):
                return self.__cache[k].value
            return None
        else:
            return self.__cache_all


cache_base = CacheBase()











# !/usr/bin/env python
# -*- coding:utf-8 -*-


"""


                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|


"""

"""统一执行测试用例，并生成测试报告"""

import os
import time

from sveltest.bin.conf import settings

from typing import Dict, Optional

from sveltest.support.common import is_dir
from sveltest.components.htmlTestRunner import HTMLTestRunner
from sveltest.support.system import ZipFile

from sveltest.support.logger_v2 import log_v2

TEST_REPORT = {
    'context_processors':
        {
            'zip': '',
            'html': '',
            'img': '',
        }
}

get_case = getattr(settings, "CASE_SUITE_PATH")


# case_path = os.path.join(get_case,'case')
# guanfl
# v1.0.1
# 20210602
# :TODO :完成第一版
class MainTestSuite(object):
    """

    """

    def __init__(self, report_filename: Optional[str] = None):
        if report_filename is None:
            self.report_filename = "自动化测试报告"
        else:
            self.report_filename = report_filename

        #     服务启动
        if getattr(settings, "API_TEST_TEMPLATE") is True:
            pass

    def test_suite_base(self, suites: Optional[list],
                        test_title_name: Optional[str] = None,
                        description: Optional[str] = None,
                        thread_count: Optional[int] = 0,
                        save_last_try: Optional[bool] = True,
                        ) -> Dict:
        """

        """

        # #测试报告保存的路径
        now = time.strftime('%Y-%m-%d', time.localtime())

        # 如果有配置html报告路径则进行使用配置里的数据、没有将进行使用tank默认的存放路径
        if not settings.TEST_REPORT:

            path = os.path.join(settings.BASE_DIR, 'report/html', now).replace('\\', '/')
        else:
            path = os.path.join(settings.TEST_REPORT["HTML"], now).replace('\\', '/')

        # 判断当 report_base 目录是否已存在、如果不存在就进行创建新的目录
        try:
            if os.path.exists(path):
                log_v2.info(path + "已存在，将跳过创建操作")
            else:
                log_v2.warning("测试结果输出存放的目录不存在，正在创建该目录")
                os.makedirs(path, exist_ok=True)
                log_v2.success(f"创建成功，创建的目录路径为：" + path)
            # else:
            #     pass

        except Exception as e:
            log_v2.error(e)
        #
        #
        joinPath = os.path.join(path, '%s_%s.html' % (self.report_filename, now)).replace("\\", "/")
        #
        fp = open(joinPath, 'wb')

        # 测试过程中的数据流写入到打开的报告中
        runner = HTMLTestRunner(
            stream=fp,
            retry=settings.TEST_CASE_ERROR_RETRY,
            title=test_title_name,
            description=description,
            save_last_try=save_last_try,
            verbosity=settings.LOGGING_VERBOSITY
        )
        #
        text_content = runner.run(suites, max_workers=thread_count)

        if settings.TEST_REPORT and settings.TEST_REPORT["START_ZIP"] is True:

            if settings.TEST_REPORT["ZIP"]:
                try:
                    log_v2.info(f"正在为你创建测试结果打包存放目录,{settings.TEST_REPORT['ZIP']}")
                    zip_file_report_path = os.path.join(settings.TEST_REPORT['ZIP'],
                                                        '%s_%s.zip' % (self.report_filename, now)).replace("\\", "/")

                    os.makedirs(settings.TEST_REPORT["ZIP"], exist_ok=True)
                    log_v2.success("创建目录成功")
                    zip_path_save = ZipFile()
                    path = zip_path_save.zip_file(path=path, zipfile_name=zip_file_report_path)
                    log_v2.info(
                        "已对目录路径为：{},下的文件进行打包成zip文件,存放路径为：{}".format(settings.TEST_REPORT["HTML"], zip_file_report_path))
                except Exception as e:
                    log_v2.error(e)

            if settings.EMAIL_CONFIG and settings.AUTO_SEND_EMAIL is True:
                log_v2.warning("已开启自动邮件发送")
                from NextTestRunner import nextEmail
                try:
                    nextEmail().send(
                        html_title="sveltest自动化测试",
                        tester=",".join(settings.TEST_REPORT['TESTER']) if settings.TEST_REPORT and
                                                                           settings.TEST_REPORT[
                                                                               'TESTER'] else "自动化测试工程师",
                        mail_title=test_title_name,
                        case_run_startTime=text_content[-1][9], case_run_time=text_content[-1][10],
                        case_run_endTime=text_content[-1][11], count=text_content[-1][0],
                        pass_count=text_content[-1][1],
                        fail_count=text_content[-1][2], error_count=text_content[-1][3], skip_count=text_content[-1][4],
                        description=description, content_title="自动化测试执行结果报告", failure_rate=text_content[-1][6],
                        pass_rate=text_content[-1][5],
                        skip_rate=text_content[-1][8],
                        error_rate=text_content[-1][7]
                    )
                except Exception as e:
                    log_v2.error(e)

        return text_content




#!/usr/bin/env python
#-*- coding:utf-8 -*-



import os
import openpyxl
from openpyxl.utils import get_column_letter,column_index_from_string

#
# TODO: 未完成
class OpenExecl(object):
    """
    用于对Excel操作、如读写
    """
    def __init__(self,file):
        self.excel = openpyxl.load_workbook(file, data_only=True)
        self.sheet = None
        self.cols = self.active().columns
        self.rows = self.active().rows
        self.filename = self.get_sheets()
        self.sheetnames = None
        self.current_sheet = self.excel.active
        self.count_cols = None
        self.max_row = self.active().max_row #行
        self.max_col = self.active().max_column #列


    def set_active(self,index):
        """
        设置当前的活动表
        :param index: 输入的是一个表格的索引值
        :return:
        """
        self.excel.active = index
        return self.excel.active


    def active(self):
        """

        :return:
        """
        return self.excel.active

    def get_sheets(self):
        return self.excel.sheetnames

    # # 切换指定的工作表
    # def switch(self,name):
    #     ws = self.excel[sheets[n]]

    # def get_cols_all(self):
    #     col = []
    #     for column in self.active().columns:
    #         cols_list = []
    #         for i in column:
    #             cols_list.append(i.value)
    #         col.append(cols_list)
    #     return col

    def get_cols_all(self):
        """

        """
        col = []
        for column in self.active().columns:
            print(column)
            cols_list = []
            for i in column:
                print(i)
                cols_list.append(i.value)
            col.append(cols_list)
        return col

    def cell(self,x,y):
        return self.excel.active.cell(x,y).value

    def get_rows_all(self,change=False):
        """
        按照行进行输出
        change : 是否对数据进行数据类型转换
        :return:
        """
        row = [i.value for rows in self.active().rows  for i in rows]
        row = []
        for rows in self.active().rows:
            row_list = [i.value for i in rows]
            row.append(row_list)
        return row

    def get_index_row(self):pass

    def get_index_col(self):pass

    def get_row(self):
        return self.active().rows

    def set_sheet_name(self,name):
        # 切换指定的工作表
        """
        drs = dr.get_sheets()
        set_sheet_name(drs[0])
        :param name:
        :return:
        """
        return self.excel[name]

    # def set_active(self):



# TODO:第一版基础功能完成 待测试
# guanfl
# v1.0
# 20210619
# def get_script(path=None, set_sheet_name=None):
#     dr = TankOpenExecl(file=r'F:\tank\src\tank\default\自动化.xlsx')
#     # 获取表单的列和行数
#     get_col_row = [dr.max_row,dr.max_col]
#
#     for d in range(len(dr.get_sheets())):
#         # 进行限制没有进行修改工作表名将进行不出来代码生成
#         if dr.get_sheets()[d][0:-1] != "Sheet":
#             # 存储所有数据
#             data_all = []
#
#             dr.set_active(d)
#
#             for c in range(0,get_col_row[0]):
#                 # 存储每一行的数据
#                 cs = []
#                 for r in range(get_col_row[1]):
#                     cs.append(dr.cell(c+1,r+1))
#                 #每一行数据进行存储到所有数据列表中
#                 data_all.append(cs)
#
#             func_list = []
#
#             with open(f'{dr.filename[d]}.py', "w+", encoding="utf-8") as f:
#
#                 for s in range(len(data_all)):
#                     if s !=0:
#                         if data_all[s][0] == "url":
#                             f.write("from selenium import webdriver\n")
#                             f.write("from selenium.webdriver.common.by import By\n")
#                             f.write("import time\n\n\n")
#
#                             f.write(f"class {dr.filename[d]}:\n\n")
#                             f.write(f"    def __init__(self):\n")
#                             url = str(data_all[s][1])
#                             try:
#                                 note = data_all[s][2]
#                                 if note:f.write(f"        #{note}\n")
#                             except:
#                                 pass
#                             f.write(f"        self.url = '{url}'\n")
#
#                         if data_all[s][0] == "open":
#                             driver = data_all[s][1]
#                             try:
#                                 note = data_all[s][2]
#                                 if note:f.write(f"        #{note}\n")
#                             except:
#                                 pass
#
#                             if driver == "chrome" or driver == "Chrome":
#                                 driver = "Chrome"
#                                 f.write(f"        self.driver = webdriver.{driver}()\n")
#
#                             if driver == "ie" or driver == "IE":
#                                 f.write(f"        self.driver = webdriver.Ie()\n")
#                             f.write(f"        self.driver.get(self.url)\n\n")
#
#                         if data_all[s][0] == "func" :
#                             func_list.append(data_all[s][1])
#                             f.write(f"    def test_{data_all[s][1]}(self):\n")
#
#
#                         if data_all[s][0] == "sleep":
#                             try:
#                                 note = data_all[s][2]
#                                 if note:f.write(f"        #{note}\n")
#                             except:
#                                 pass
#                             f.write(f"        time.sleep(4)\n")
#
#                         if data_all[s][0] == "input":
#
#
#                             try:
#                                 note = data_all[s][3]
#                                 if note:f.write(f"        #{note}\n")
#                             except:
#                                 pass
#                                 # upper() #转换成大写
#                             element_by = str(data_all[s][1]).split("=")
#                             f.write(f"        self.driver.find_element(by=By.{element_by[0].upper()},value='{element_by[1]}').send_keys('{data_all[s][2]}')\n")
#
#
#                         if data_all[s][0] == "end":
#
#
#                             try:
#                                 note = data_all[s][3]
#                                 if note:f.write(f"        #{note}\n")
#                             except:
#                                 pass
#
#                             try:
#                                 f.write(f"\n\n\nif __name__ == '__main__':\n    case = {dr.filename[d]}()\n    case.test_{func_list[0]}()\n")
#                             except:
#                                 print("不允许在没有声明方法前进行使用end关键字")
#                             break
#
#






import os
import functools
import unittest

from sveltest.components.dblib import ShelveBase


def rely(case_name:str=None):
    """
    :param case
    :return:
    """
    def wrapper_case(test_func):

        @functools.wraps(test_func)
        def _func(cur):

            if case_name == test_func.__name__:
                raise ValueError("{} 你不能将依赖作为自己".format(case_name))

            failures = [str(fail_[0]).split("=")[0].split(" ")[0] for fail_ in cur._outcome.result.failures]

            errors = [str(fail_[0]).split("=")[0].split(" ")[0] for fail_ in cur._outcome.result.errors]

            skipped = [str(fail_[0]).split("=")[0].split(" ")[0] for fail_ in cur._outcome.result.skipped]

            tag_status = (case_name in failures) or (case_name in errors) or (case_name in skipped)

            test = unittest.skipIf(tag_status, '{} ({} rely {})'.format(test_func.__name__,test_func.__name__,case_name))(test_func)
            return test(cur)
        return _func
    return wrapper_case


import inspect

def env(arg=None,**kwargs):
    def func(cls):
        def wrapper(**kwargs):

            if kwargs:
                from sveltest.support.common import ObjectDict
                print(cls.__name__)

                kw = ObjectDict(kwargs)
                cls.cls_name = cls.__name__

                try:
                    from sveltest.bin.conf import settings
                    cls.obj_conf = ObjectDict(settings.ENVIRONMENT_CLASSES_CONFIG)
                except:
                    pass

                cls.env =  cls.obj_conf.DEFAULT_ENVIRONMENT_NAME

                try:
                    #:todo 暂时不进行该逻辑处理，后面版本将完善
                    # if settings.DEBUG:
                    #      cls.ENV_CLASS = "ENVIRONMENT_CLASS_DEV"
                    # else:
                    #     cls.ENV_CLASS = "ENVIRONMENT_CLASS_PROD"


                    cls.frontend_env_name = cls.obj_conf.FRONTEND
                    cls.backend_env_name = cls.obj_conf.BACKEND
                    if cls.env == "dev": cls.ENV_CLASS = "ENVIRONMENT_CLASS_DEV"
                    if cls.env == "sit": cls.ENV_CLASS = "ENVIRONMENT_CLASS_SIT"
                    if cls.env == "uat": cls.ENV_CLASS = "ENVIRONMENT_CLASS_UAT"
                    if cls.env == "prod": cls.ENV_CLASS = "ENVIRONMENT_CLASS_PROD"


                    cls.env_frontend_info = ObjectDict(cls.obj_conf.FRONTEND[cls.ENV_CLASS])
                    cls.env_backend_info = ObjectDict(cls.obj_conf.BACKEND[cls.ENV_CLASS])


                    cls.env_backend_port = cls.env_backend_info.ENVIRONMENT_PORT

                    cls.env_backend_host = cls.env_backend_info.ENVIRONMENT_HOST

                    if not cls.env_backend_info:
                        cls.env_backend_network_protocol = cls.env_backend_info.NETWORK_PROTOCOL
                    else:
                        raise Exception("请将 ENVIRONMENT_CLASSES_CONFIG 配置完整")


                    cls.env_frontend_port = cls.env_frontend_info.ENVIRONMENT_PORT
                    cls.env_frontend_host = cls.env_frontend_info.ENVIRONMENT_HOST
                    cls.env_frontend_network_protocol = cls.env_frontend_info.NETWORK_PROTOCOL

                    if  cls.env_frontend_network_protocol =="http" : protocol = "http://"
                    if  cls.env_frontend_network_protocol =="https" : protocol = "https://"
                    if  cls.env_frontend_network_protocol =="ws" : protocol = "ws://"
                    if  cls.env_frontend_network_protocol =="wss" : protocol = "wss://"

                    url_ = ''.join([protocol,cls.env_frontend_host])
                    if cls.env_frontend_port is not None and cls.env_frontend_port!="":
                        cls.frontend_host = ":".join([url_,str(cls.env_frontend_port)])
                    else:
                        cls.frontend_host = url_


                    try:
                        url_end = ''.join([protocol,cls.env_backend_host])
                        if cls.env_backend_port is not None and cls.env_backend_port!="":
                            cls.backend_host = ":".join([url_end,str(cls.env_backend_port)])
                        else:
                            cls.backend_host = url_end
                    except:
                        cls.backend_host = None


                except:
                    cls.current_env_host = cls.obj_conf.ENVIRONMENT_HOST if cls.obj_conf.ENVIRONMENT_HOST else "80"
                    cls.current_env_port = cls.obj_conf.ENVIRONMENT_PORT

                return cls


        return wrapper(**kwargs)

    return func(arg) if inspect.isclass(arg) else func



def request_env(arg=None,**kwargs):
    def func(cls):

        def wrapper(**kwargs):
            from sveltest.support.common import ObjectDict

            cls.params = ObjectDict(kwargs)

            cls.cls_name = cls.__name__


            try:
                from sveltest.bin.conf import settings

                cls.obj_conf = ObjectDict(settings.ENVIRONMENT_CLASSES_CONFIG)
            except:
                pass

            cls.env =  cls.obj_conf.DEFAULT_ENVIRONMENT_NAME
            cls.env_headers =  cls.obj_conf.HEADERS

            if cls.env == "dev": cls.ENV_CLASS = "ENVIRONMENT_DEV_HOST"
            if cls.env == "sit": cls.ENV_CLASS = "ENVIRONMENT_SIT_HOST"
            if cls.env == "uat": cls.ENV_CLASS = "ENVIRONMENT_UAT_HOST"
            if cls.env == "prod": cls.ENV_CLASS = "ENVIRONMENT_PROD_HOST"

            cls.info = ObjectDict(cls.obj_conf)
            cls.env_ = ObjectDict(cls.info.FRONTEND)


            # try:
            #     #:todo 暂时不进行该逻辑处理，后面版本将完善
            #     # if settings.DEBUG:
            #     #      cls.ENV_CLASS = "ENVIRONMENT_CLASS_DEV"
            #     # else:
            #     #     cls.ENV_CLASS = "ENVIRONMENT_CLASS_PROD"
            #
            #
            cls.frontend_env_host = cls.env_[cls.ENV_CLASS]

            cls.backend_env_host = cls.env_[cls.ENV_CLASS]
            # print( bool(cls.env_headers))
            cls.Header = {
                "User-Agent": "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/104.0.0.0 Safari/537.36"
            }

            pub = ShelveBase()
            if pub.get("token", ):
                cls.token_ = cls.Header.update(pub.get("token", ))
                pub.quit()

            if bool(cls.env_headers) :
                cls.Header = cls.env_headers

            else:
                if cls.cls_name.upper() in ["POST", "PUT", "PATCH"]:

                    cls.Header.update({
                        'accept': "application/json", "Content-Type": "application/json",

                    })
            return cls
        return wrapper(**kwargs)

    return func(arg) if inspect.isclass(arg) else func



# http://www.66ip.cn/pt.html
# 代理器
# def proxy(=None,**kwargs):
#
#     def func(cls):
#
#         def wrapper(*args, **kwargs):
#             print('获取关键字参数的dict', kwargs)
#             return cls(*args, **kwargs)
#         return wrapper
#     return func


proxy_manager=["https://proxy.ip3366.net/doc/putong/"]






#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2021/5/28


import base64
import os

import redis
from typing import Tuple, List, Dict, Union


# TODO: 未完成
class Base64(object):

    def __init__(self,imgPath):
        self.imgPath = imgPath


    def encode(self):
        pass

    def decode(self):
        pass



    def save(self,filePath=None):
        """
        :param filePath:
        :return:
        """
        with open(self.imgPath,"rb") as f:#转为二进制格式
            base64_data = base64.b64encode(f.read())#使用base64进行加密
            file=open(filePath,'wt')
            #写成文本格式
            file.write(base64_data.decode())
            file.close()





class FastRedis:
    def __init__(self,host:str="127.0.0.1",port:Union[int,str]=6379,
                 db:str=0,decode_responses:bool=True,password:Union[str,int]=None
                 ):
        self.pool =  redis.ConnectionPool(
            host=host, port=port,
            password=password,db=db,
            decode_responses=decode_responses
        )


    def _connect(self):
        # self.pool = redis.Redis(self.pool,host='localhost', port=6379, db=0,charset="utf-8")
        return redis.Redis(connection_pool=self.pool)


    def flushall(self):
        """清空redis"""
        return self._connect().flushall()

    def native(self):
        """
        原生redis
        """
        return redis.Redis(connection_pool=self.pool)


fa =FastRedis()
# fa.()



class StringRedis(FastRedis):
    def __init__(self):
        super(StringRedis,self).__init__()
        self.redis = self._connect()


    def set(self,key:Union[int,str,float,bool],value:Union[Dict,int,str,float,bool]) -> Union[Dict,int]:
        """设置字符串
        nx:为TRUE时则会进行将name的value进行更换
        """
        if self.redis.set(key, value, nx=True):
            return True
        else:
            return {"status":0,"message":f"{key}已存在数据库"}
#
#     def get(self,key):
#         return self.redis.get(key)
#
#     def null_set(self,key,value):
#         """当key不存在为True"""
#         return self.redis.setnx(key, value)
#
#     def setTime(self,key,value,time):
#         """指定时间key会变为null
#         time 可以为s 可以为timedate对象
#         """
#         return self.redis.setex(key,time,value)
#
#     def sets(self,*args, **kwargs):
#         """一次设置多个键
#         1、k1=1
#         2、{k1:"1"}
#         """
#         return self.redis.mset(*args, **kwargs)
#     def gets(self,*args, **kwargs):
#         """一次设置多个键
#         1、k1=1
#         2、gets("k2","set")
#         """
#         return self.redis.mget(*args, **kwargs)
#
#     def reset(self,ke, value):
#         """设置新的值、获取原来的值"""
#         return self.redis.getset(ke, value)
#
#     def getBit(self,key,start,stop):
#         """安装字节进行获取值"""
#         return self.redis.getrange(key,start,stop)
#
#     def length(self,key):
#         """获取key的字节长度、中文为3个字符"""
#         return self.redis.strlen(key)
#
#     def autoCount(self,key,count,type=0):
#         """key值自增"""
#         if type == 0:
#             return self.redis.incr(key,count)
#         else:
#             return self.redis.incrbyfloat(key,count)
#
#     def autoDecr(self,key,count,type=0):
#         """key值自减"""
#         if type == 0:
#             return self.redis.decr(key,count)
#         else:
#             return self.redis.incrbyfloat(key,count)
#
#
#     def addString(self,key,text):
#         """key值追加字符串"""
#         return self.redis.append(key,text)
#
# class HashRedis(TankRedis):
#     def __init__(self):
#         super(HashRedis,self).__init__()
#         self.redis = self.connect()
#
#
#     def hmset(self,key,field,value):
#        """ 单个增加 - -修改(单个取出) - -没有就新增，有的话就修改"""
#        return self.redis.hset(key,field,value)
#
#
#     def hget(self,key,field):
#         return self.redis.hget(key, field)
#
#     def getKeys(self,key):
#         return self.redis.hkeys(key)
#
#     def keys(self,name, keys, *args):
#         return self.redis.hmget(name, keys, *args)
#
#     def add(self,name, key, value):
#         """只能新建"""
#         return self.redis.hsetnx(name,key,value)
#
#     def setAdds(self,name, set_:dict):
#         """添加字典类型"""
#         return self.redis.hmset(name,set_)
#
#     def getAll(self,name):
#         return self.redis.hgetall(name)
#
#     def length(self,name):
#         """hlen(name) 获取键值对长度"""
#         return self.redis.hlen(name)
#
#     def values(self,name):
#         """获取所有值"""
#         return self.redis.hvals(name)
#
#     def ists(self,name,key):
#         """查询某个key是否存在"""
#         return self.redis.hexists(name, key)
#
#     def delKey(self,name,*keys):
#         """删除指定的键值对"""
#         return self.redis.hdel(name,*keys)
#
#
#     def autoCount(self,name,key,count):
#         """key值自增减 int"""
#         return self.redis.hincrby(name,key,amount=count)
#
#     def autoFloat(self,name,key,count):
#         """key值自增减  float"""
#
#         return self.redis.hincrbyfloat(name,key,count)



#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2022/10/17


from win10toast import ToastNotifier
import _thread
from time import sleep
from typing import Optional,Sequence,Union


"""
win10toast 的库底层为win32api、win32con和win32gui
"""

toaster = ToastNotifier()


def show_toast(title,msg):
    toaster.show_toast(title=title,
                       msg=msg,
                       icon_path=None,
                       duration=1,
                       threaded=True
                       )


def toast_send(title:Sequence[str],msg:Optional[str]=None):
    _thread.start_new_thread(show_toast, (title,msg))
    sleep(0.1)



def test_toast(count:Union[str,int]=0,
               pass_count:Union[str,int]=0,
               error_count:Union[str,int]=0,
               fail_count:Union[str,int]=0,
               skip_count:Union[str,int]=0,
               ):
    toast_send(title="sveltest-测试执行结果",msg=f"""
    你本次执行的测试用例数为 {count} 条，其具体结果如下：
    通过 {pass_count}条，失败 {fail_count}条，
    异常 {error_count}条，跳过 {skip_count}条，
    """)

if __name__ == '__main__':
    test_toast()






import setuptools  #导入工具包

# with open("requirements.txt") as fin:
#     REQUIRED_PACKAGES = fin.read()
from sveltest.version import version

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()
# 读写README.md 文件

setuptools.setup(
    name="sveltest",  #项目名称
    version=version, #项目版本
    author="guanfl", #开发者名称
    author_email="gfl13453001@163.com",  #邮箱
    description="高效率测试开发集成框架",  #描述
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/sveltest-Team/sveltest", #github地址
    packages = setuptools.find_packages(),  # 包含所有src中的包
    # package_dir = {'':'sveltest'},   # 告诉distutils包都在src下

    classifiers=[
        # 3 - Alpha
        # 4 - Beta
        # 5 - Production/Stable
        "Development Status :: 4 - Beta",
        # 支持的python版本
        # "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ], #依赖环境

    # python 依赖版本
    python_requires='>=3.6',

    # Appium-Python-Client==0.49
    # certifi==2019.11.28



    # chardet==3.0.4
    # decorator==4.4.1·
    # facebook-wda==0.4.2
    #
    # Pillow==6.2.1


    # pytesseract==0.3.1
    # python-dateutil==2.8.1
    # requests==2.22.0
    # retry==0.9.2
    # selenium==3.141.0
    # six==1.13.0
    # testdata==1.1.3
    # text-unidecode==1.3
    # urllib3==1.25.7

    install_requires = [
        # 'Appium-Python-Client==0.49',
        'selenium',
        'testdata==1.1.3',
        'PyMySQL==0.9.3',
        'Faker==3.0.0',
        'flanker',
        # 'fastapi==0.65.2',
        # 'uvicorn==0.14.0',
        'requests==2.26.0',
        'rich==10.12.0',
        'yagmail',
        'jinja2',
        # 'gooey==1.0.8.1',
        'pyyaml',
        'NextTestRunner',
        'win10toast',
        'loguru==0.6.0',
        'xmltodict',

    ],#第三方依赖包
    package_data={
            #任何包中含有.txt文件，都包含它
            '': ['*.py','*.py-tpl','*.json','*.html','*.pyc','*.xlsx',"*.*"],
            #包含demo包data文件夹中的 *.dat文件

        },
    keywords = 'sveltest linux python selenium unittest',

    # 命令參數 slt 命令会自动执行指定文件
    entry_points={
            'console_scripts': [
                # 命令 = 包.模块.方法
                'slt=sveltest.bin.conf.base.sveltest_main:main'
            ],
        },

)











#!/usr/bin/env python
#-*- coding:utf-8 -*-

# authors:guanfl
# 2022/9/26

import datetime
import json
import smtplib
import poplib as pop
import yagmail
from email.mime.application import MIMEApplication
from email.mime.image import MIMEImage
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from typing import Optional,Union,Dict,List,NoReturn

from flanker import mime

from sveltest.support.logger_v2 import log_v2

class SveltestEmail:
    # user 代表用户名
    # password 代表邮箱密码
    # host 代表发信服务器
    # port 发信端口
    # smtp_ssl 使用ssl协议（默认为true）
    # encoding 编码方式（默认utf-8）

    def __init__(self,user:Optional[str],password:Optional[str],
                 host:Optional[str]="smtp.163.com",
                 ):
        """

        :param user:
        :param password:
        :param host:
        """
        self.svelte_email = yagmail.SMTP(user=user, password=password, host=host)

    def img(self,path:Optional[str]=None):
        return yagmail.inline(path)


    def send(self,to:Union[str,Dict[str,str],List]=None,
             cc:Union[str,Dict[str,str],List]=None,
             bcc:Union[str,Dict[str,str],List]=None,
             subject:Union[str,Dict[str,str]]=None,
             contents:Union[List,str]=None,
             preview_only:Optional[bool]=None,
             attachments:Optional[List]=None,
             prettify_html:Optional[bool]=True,
             headers:Optional[Dict]=None,
             ) -> NoReturn :
        """

        :param to:
        :param cc:
        :param bcc:
        :param subject:
        :param contents:
        :param preview_only:
        :param attachments:
        :param prettify_html:
        :param headers:
        :return:
        """

        # 发送邮件
        self.svelte_email.send(
            # to 收件人，如果一个收件人用字符串，多个收件人用列表即可
            to=to,
            # cc 抄送，含义和传统抄送一致，使用方法和to 参数一致
            cc=cc,
            # subject 邮件主题（也称为标题）
            subject=subject,
            # contents 邮件正文
            contents=contents,
            # 仅预览，可发送邮件但对方收不到
            preview_only=preview_only,
            # 头部信息，传递字典参数
            headers=headers,
            prettify_html=prettify_html,
            bcc=bcc,#密送，与抄送的区别是收件人看不到其他人的邮箱地址
            # attachments 附件，和收件人一致，如果一个附件用字符串，多个附件用列表
            attachments=attachments)

        # 记得关掉链接，避免浪费资源
        self.svelte_email.close()




class Mail:
    """邮件的封装
    Attributes:
    """
    def sender(self,sender,password,toaddrs,title,content=None):
        """发送的基本配置
        Args:
            sender:发送人的邮箱
            password:发送人的邮件授权码不是邮箱密码
            toaddrs:接收人的邮箱可以是多个，列表类型
            title:邮件的标题
            content:邮件的正文
        Returns:
            example:
        Raises:
        """
        log_v2.info("正在发送自动邮件...")

        try:
            self.sender = sender

            self.password = password

            self.toaddrs = toaddrs

            self.content = content

            self.textApart = MIMEText(self.content,"html", 'utf-8')
            self.m = MIMEMultipart()
        # 加上这个qq邮箱不会退邮箱
            self.m['Subject'] = title
            self.m['from'] = sender
        # 用于列表元素切割
            self.m['to'] = ",".join(self.toaddrs)
            self.m.attach(MIMEText(self.content, 'html', 'utf-8'))
            # message = MIMEText(mail_msg, 'html', 'utf-8')


        except Exception:
            log_v2.error("邮件配置信息有误请重新修改邮件配置后再重试发送邮件操作")

    def html(self,path,filename):
        """发送正文为HTML格式的邮件
        Args:
            path:html文件路径
            filename:文件名
        Returns:
            example:
        Raises:
        """
        filepath = path
        htmlApart = MIMEImage(open(filepath, 'rb').read(), filepath.split('.')[-1])
        htmlApart.add_header('Content-Disposition', 'attachment', filename=filename)
        self.m.attach(htmlApart)

        return htmlApart
    def universalFile(self,path,filename):
        """发送任意类型的邮件格式
        Args:
            path:文件路径
            filename:文件名称
        Returns:
            example:
        Raises:
        """
        pdfFile = path
        pdfApart = MIMEApplication(open(pdfFile, 'rb').read())
        pdfApart.add_header('Content-Disposition', 'attachment', filename=filename)
        self.m.attach(pdfApart)

    def smtp(self,smtp='smtp.163.com'):
        """邮件发送
        Args:
            smtp:邮件的服务类型
        Returns:
            example:
        Raises:
        """
        try:
            server = smtplib.SMTP_SSL(smtp)
            server.login(self.sender, self.password)
            server.sendmail(self.sender, self.toaddrs, self.m.as_string())
            log_v2.info("邮件已发送成功")
            server.quit()
        except smtplib.SMTPException as e:
            log_v2.info(e)

class UnittestMail:
    """
        发送邮件的整个流程封装
    Attributes:
        url:传入的是一个接口地址
    """
    def __init__(self,sender,password,toaddrs,title,htmlPATH,FILEname,content=None):
        """
        Args:
            sender:
            password:
            toaddrs:
            title:
            htmlPATH:
            FILEname:
            content:
        Returns:
            example:
        Raises:
        """
        try:
            log_v2.info("本次邮件发送人用户名：%s，密码：******"%sender)
            log_v2.info("本次邮件接收人用户名：%s"%toaddrs)
            e = Mail()
            e.sender(sender=sender,password=password,toaddrs=toaddrs,title=title,content=content)
            log_v2.info("邮件主题为%s"%title)
            e.universalFile(htmlPATH,FILEname)
            log_v2.info("邮件正文为%s" % content)
            log_v2.info("邮件附件路径%s" % htmlPATH)
            e.smtp()
        except Exception as e:
            log_v2.error("请往框架配置文件中修改邮件配置信息,error -- %s"%e)


class DateEncoder(json.JSONEncoder):
    """处理序列化时时间格式无法进行转json 这里重写了JSONEncoder类"""
    def default(self, obj):
        if isinstance(obj, datetime.datetime):
            return obj.strftime('%Y-%m-%d %H:%M:%S')

        elif isinstance(obj, datetime.date):
            return obj.strftime("%Y-%m-%d")

        else:
            return json.JSONEncoder.default(self, obj)

class PopEmailBase:
    """
    邮件处理的主逻辑类
    """

    def __init__(self,host="pop.163.com"):
        """

        """
        self.pop_session = pop.POP3_SSL(host=host)
        self.content_data = None

    def connect(self,user=None, pwd=None):
        """
        进行对pop服务器连接操作
        :param user: 邮件用户名
        :param pwd:第三方客户端授权码
        :return:返回pop实例
        """
        self.pop_session.user(user)
        self.pop_session.pass_(pwd)
        log_v2.info(f"user: {user}")

        return self.pop_session

    def get_list(self):
        """获取邮件列表"""
        return self.pop_session.list()

    def stat(self):
        """

        """
        return self.pop_session.stat()

    def get_index(self):
        """

        """
        result = []
        data = self.pop_session.list()
        for x in data[1]:
            d = x.decode("utf-8").split(" ")
            result.append({"id":d[0],"size":d[1]})
        return result

    def content(self,index):
        return self.pop_session.retr(index)

    def str_from(self,content):
        """

        """
        msg_bytes_content = b'\r\n'.join(content[1])
        self.content_data = mime.from_string(msg_bytes_content)
        # print(data.content_type.is_singlepart()) #判断类型
        return self.content_data

    def header(self):
        """

        """
        r = {}
        for data in self.content_data.headers.items():
            r[data[0]] = data[1]
        return r

    def contentEml(self,eml):
        """

        """
        # 判断是否为单部分
        if eml.content_type.is_singlepart():
            eml_body = eml.body
        else:
            eml_body = ''
            for part in eml.parts:
                # 判断是否是多部分
                if part.content_type.is_multipart():
                    eml_body = self.contentEml(part)
                else:
                    if part.content_type.main == 'text':
                        eml_body = part.body
        return eml_body

    def body(self):
        """

        """

        if  self.content_data.content_type.is_singlepart():
            return self.content_data.body
            pass
        else:
            for part in  self.content_data.parts:
                if part.content_type.is_multipart():
                    eml_body = self.contentEml(part)
                    return eml_body
                if part.content_type.is_message_container():
                    return part
                if part.content_type.is_singlepart():
                    if part.content_type.main == "text":
                        return part.body

    def close(self):
        self.pop_session.quit()





#!/usr/bin/env python
#-*- coding:utf-8 -*-

"""
                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|

"""

import os
import sys

BASE_DIR = os.path.dirname(__file__)
sys.path.insert(0,BASE_DIR)



__all__ = [

    'svelteTestLoader',
    'TestCase',
    'SvelteTestResult',
    'SvelteTextTestRunner',
    'main',

    'SkipTest',
    'skip',
    'skipIf',
    'skipUnless',
    'expectedFailure',

    'parameterized','char','extends','serialization','Data','FileData','unpack',
    'MockChar','mysql',

]

from sveltest.runner import main

from sveltest.runner import (
    SvelteTestResult,SvelteTextTestRunner
)

from sveltest.loader import (
    svelteTestLoader
)

from sveltest.case import (TestCase,HttpTestCase)



from unittest import (
    skip,skipIf,skipUnless,expectedFailure,SkipTest
)


from sveltest.components._api._parameterized import (
unpack,char,MockChar,extends,serialization,FileData,mysql,parameterized
)

# from sveltest.components._test_core import (rely,env,request_env)

from .components.web.base import  (PageBase,PageBaseObject,WebActon)

from .components.network.main import (HttpResponseHeaders,RequestBase)



#!/usr/bin/env python
#-*- coding:utf-8 -*-
"""


                   _  _              _
                  | || |            | |
  ___ __   __ ___ | || |_  ___  ___ | |_
 / __|\ \ / // _ \| || __|/ _ \/ __|| __|
 \__ \ \ V /|  __/| || |_|  __/\__ \| |_
 |___/  \_/  \___||_| \__|\___||___/ \__|


"""
import functools
import importlib
import unittest
from fnmatch import fnmatchcase
from sveltest.bin.conf import settings
from sveltest.case import TestCase
from typing import (Optional, Any, Tuple, Dict)


class Loader(unittest.TestLoader):

    """
    该类负责根据各种标准加载测试并将它们封装在测试套件中返回
    """
    testNamePatterns = None

    testMethodPrefix = 'test' if settings.TEST_METHOD_PREFIX == "test" else settings.TEST_METHOD_PREFIX
    CaseListInfo = []

    def getTestCaseNames(self, testCaseClass:Optional[Any]):
        """返回在testCaseClass中找到的方法名的排序序列
        """


        def shouldIncludeMethod(attrname):
            if not attrname.startswith(self.testMethodPrefix):
                return False
            testFunc = getattr(testCaseClass, attrname)
            if not callable(testFunc):
                return False
            fullName = f'%s.%s.%s' % (
                testCaseClass.__module__, testCaseClass.__qualname__, attrname
            )
            if settings.COLLECT_STATUS:
                case_info = {"module": testCaseClass.__module__,"class": {"testcase": testCaseClass.__name__,"declare": testCaseClass.__doc__},
                    "method": {"testcase": attrname,"declare": testFunc.__doc__}}

                self.CaseListInfo.append(case_info)

            return self.testNamePatterns is None or \
                   any(fnmatchcase(fullName, pattern) for pattern in self.testNamePatterns)

        testFnNames = list(filter(shouldIncludeMethod, dir(testCaseClass)))

        if self.sortTestMethodsUsing:
            testFnNames.sort(key=functools.cmp_to_key(self.sortTestMethodsUsing))
        return testFnNames




    def load_tests_module(self, module:Optional[str],
                          *args:Optional[Tuple],
                          pattern:Optional[str]=None,
                          **kws:Optional[Dict]
                          ) -> object:
        """返回给定模块中包含的所有测试用例的集合(暂未实现)"""


        # if len(args) > 0 or 'use_load_tests' in kws:
        #     warnings.warn('use_load_tests is deprecated and ignored',
        #                   DeprecationWarning)
        #     kws.pop('use_load_tests', None)
        # if len(args) > 1:
        #     # Complain about the number of arguments, but don't forget the
        #     # required `module` argument.
        #     complaint = len(args) + 1
        #     raise TypeError('loadTestsFromModule() takes 1 positional argument but {} were given'.format(complaint))
        # if len(kws) != 0:
        #     # Since the keyword arguments are unsorted (see PEP 468), just
        #     # pick the alphabetically sorted first argument to complain about,
        #     # if multiple were given.  At least the error message will be
        #     # predictable.
        #     complaint = sorted(kws)[0]
        #     raise TypeError("loadTestsFromModule() got an unexpected keyword argument '{}'".format(complaint))

        test_module = importlib.import_module(module)

        # 存储排序好的测试用例
        tests = []
        # 进行加载当前的模块下的所有属性及方法
        for name in dir(test_module):

            obj = getattr(test_module, name)


            # 判断是否是一个对象且进行继承至testcase
            # if isinstance(obj, type) and issubclass(obj, unittest.case.TestCase) :
            if isinstance(obj, type) and issubclass(obj, TestCase) :
                # 如果被继承了那么则将这个对象加入tests列表中
                tests.append(self.loadTestsFromTestCase(obj))

        # load_tests = getattr(module, 'load_tests', None)
        # 将测试用例传入测试用例集中
        tests = self.suiteClass(tests)
        # if load_tests is not None:
        #     try:
        #         return load_tests(self, tests, pattern)
        #     except Exception as e:
        #         error_case, error_message = _make_failed_load_tests(
        #             module.__name__, e, self.suiteClass)
        #         self.errors.append(error_message)
        #         return error_case
        return tests

class PlatformOperationLoader(Loader):

    """
    用于对测试平台化的支持
    """







svelteTestLoader = Loader()
